@@ -136,7 +136,12 @@ void time_decode(unsigned long long bits, unsigned int samples)
 		vlprintf(1, "mktime failed\n");
 		goto error;
 	}
-	t -= timezone;
+/* XXX FreeBSD takes care of timezone internally, I don't know about others. 
+ * -db (VA3DB)
+ */
+#ifdef __linux__
+ 	t -= timezone;
+#endif
 /*	t -= tz == ZONE_MESZ ? 7200 : 3600; */
 /* bug? isn't summer time later than winter time ? */
 	t -= tz == ZONE_MESZ ? 3600 : 7200 ; 
