@@ -32,6 +32,7 @@
 #include <assert.h>
 #include <syslog.h>
 #include <string.h>
+#include <sys/types.h>
 
 #include "fskl1.h"
 #include "fskutil.h"
@@ -481,7 +482,6 @@ static void fec_rx(void)
 	int trk = 0;
 #endif /* FREQ_TRACKING */
 
-	modefamily = FSK;
 	l1_fsk_clear_requests();
 	errprintf(SEV_INFO, "mode: amtor fec rx\n");
 	kbd_clear_and_fill(NULL, 0);
@@ -781,7 +781,6 @@ static void arq_statemachine(void)
 {
 	int eflg, i, rq;
 
-	modefamily = FSK;
 	l1_fsk_clear_requests();
 	kbd_clear_and_fill(NULL, 0);
 	as.flags = 0;
@@ -1185,7 +1184,6 @@ static void fec_tx(int sel, const char *mode)
 	unsigned char *bp;
 	unsigned short txch;
 	
-        modefamily = FSK;
 	l1_fsk_clear_requests();
 	kbd_clear_and_fill(NULL, 0);
 	as.flags = 0;
@@ -1382,7 +1380,6 @@ void *mode_amtor_arq(void *dummy)
 {
 	int i;
 
-	modefamily = FSK;
 	l1_fsk_clear_requests();
 	errprintf(SEV_INFO, "mode: amtor arq call\n");
 	i = master_call();
@@ -1396,7 +1393,6 @@ void *mode_amtor_arq(void *dummy)
 
 void *mode_amtor_colfec(void *dummy)
 {
-	modefamily = FSK;
 	errprintf(SEV_INFO, "mode: amtor colfec\n");
 	fec_tx(0, "AMTOR COLLECTIVE FEC TRANSMIT");
 	return NULL;
@@ -1404,7 +1400,6 @@ void *mode_amtor_colfec(void *dummy)
 
 void *mode_amtor_selfec(void *dummy)
 {
-	modefamily = FSK;
 	errprintf(SEV_INFO, "mode: amtor selfec\n");
 	fec_tx(1, "AMTOR SELECTIVE FEC TRANSMIT");
 	return NULL;
