@@ -33,6 +33,7 @@
 #include <syslog.h>
 #include <string.h>
 #include <stdio.h>
+#include <sys/types.h>
 
 #include "fskl1.h"
 #include "fskutil.h"
@@ -64,9 +65,10 @@ static const char rtty_chars[65] = LETTERS NUMBERS_DL;
 /* --------------------------------------------------------------------- */
 
 static struct {
-	unsigned int baud;
+	//unsigned int baud;
+	float baud;
 	int rxinvert, txinvert;
-} rp = { 45, 0, 0 };
+} rp = { 45.45, 0, 0 };
 
 static struct {
 	unsigned int figmode;
@@ -89,9 +91,10 @@ static unsigned char txchars[NUMTXCHARS];
 
 /* --------------------------------------------------------------------- */
 
-void rtty_set_params(unsigned int baud, int rxinvert, int txinvert)
+//void rtty_set_params(unsigned int baud, int rxinvert, int txinvert)
+void rtty_set_params(float baud, int rxinvert, int txinvert)
 {
-	errprintf(SEV_INFO, "rtty params: baud %u invert%s%s\n",
+	errprintf(SEV_INFO, "rtty params: baud %3.2f invert%s%s\n",
 		  baud, rxinvert ? " RX" : "", txinvert ? " TX" : "");
 	if (baud >= 10 && baud <= 1000)
 		rp.baud = baud;
@@ -121,7 +124,6 @@ void *mode_rtty_rx(void *dummy)
 	int deaf = 0;
 	unsigned char rxchar;
 
-	modefamily = FSK;
        	l1_fsk_clear_requests();
 	errprintf(SEV_INFO, "mode: rtty rx\n");
 //	bufprintf(HFAPP_MSG_DATA_STATUS, "RTTY RX %u BAUD LETTERS", rp.baud);
@@ -168,7 +170,7 @@ void *mode_rtty_rx(void *dummy)
 			    bufprintf(HFAPP_MSG_DATA_RECEIVE, "%c", 
 				rtty_chars[rxchar + rs.figmode]);
 //			bufprintf(HFAPP_MSG_DATA_STATUS, 
-//			    "RTTY RX %u BAUD %s", rp.baud, rs.figmode ? 
+//			    "RTTY RX %f BAUD %s", rp.baud, rs.figmode ? 
 //			    "FIGURES" : "LETTERS");
 			}
 		}
@@ -187,7 +189,7 @@ static unsigned char rtty_getcharbits(void)
 	unsigned short ch;
 	char *diddle = "LTRS "; 
 	int i, diddle_start = 35, eof_ack_start = 5; 
-	/*  We wait for the first kbd_ack() for eof_ack_start times `KBD_EOF` 
+	/*  We wait with giving the first kbd_ack() for eof_ack_start times `KBD_EOF` 
 	    to prevent a race condition between the autorx() function of hfterm 
 	    (in rxtx.c) and the real transmit here which lasts some time.
 	   
@@ -262,10 +264,9 @@ void *mode_rtty_tx(void *dummy)
 	l1_time_t tm_chinc = 7500000/rp.baud;
 	int i;
 
-      	modefamily = FSK;
        	l1_fsk_clear_requests();
 	errprintf(SEV_INFO, "mode: rtty tx\n");
-	bufprintf(HFAPP_MSG_DATA_STATUS, "RTTY TX %u BAUD LETTERS", rp.baud);
+	bufprintf(HFAPP_MSG_DATA_STATUS, "RTTY TX %3.2f BAUD LETTERS", rp.baud);
 	rs.idle_cnt = IDLE_SHIFT_FREQ;
 	rs.figmode = 0;
 	memset(txchars, 0xff, sizeof(txchars));
@@ -291,7 +292,7 @@ void *mode_rtty_tx(void *dummy)
 		    (txtimes[i], tm_inc, 0, rp.txinvert, i, 8, txchars+i);
 //		if (i == 0)
 // 			bufprintf(HFAPP_MSG_DATA_STATUS, 
-//			"RTTY TX %u BAUD %s", rp.baud, rs.figmode ? "FIGURES" : "LETTERS");
+//			"RTTY TX %f BAUD %s", rp.baud, rs.figmode ? "FIGURES" : "LETTERS");
 // i tried to put it here for autorx to work but... see above
 //		kbd_ack();
 	}
