@@ -30,7 +30,7 @@
 #include "config.h"
 #endif
 
-#include <asm/byteorder.h>
+//#include <asm/byteorder.h>
 #include <sys/soundcard.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -44,7 +44,7 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include <signal.h>
-#include <endian.h>
+//#include <endian.h>
 #include <pthread.h>
 #include <syslog.h>
 
@@ -73,6 +73,8 @@ int brakepoint = 70;
 // flag: fdx possible, hdx chosen by option
 int fdx_possible_hdx_chosen = 0;
 
+
+
 union {
 	caddr_t v;
 	short *s;
@@ -81,7 +83,11 @@ union {
 	caddr_t v;
 	short *s;
 } obuf;
-unsigned int isize, osize;
+union {
+	caddr_t v;
+	short *s;
+} buf;
+unsigned int size, isize, osize;
 
 /* --------------------------------------------------------------------- */
 
@@ -94,7 +100,7 @@ unsigned int isize, osize;
 #ifndef MAP_FAILED
 #define MAP_FAILED ((caddr_t)-1)
 #endif
-
+#define EXCESS_FRAGS 3
 #define NTESTS 4
 #define FULL_MMAP
 
@@ -107,7 +113,11 @@ int probe_oss(const char *name_audio)
         unsigned int times[NTESTS];
         unsigned int isize;
         caddr_t ibuf, ibufe;
-        unsigned int avg;
+        union {
+	    caddr_t v;
+	    short *s;
+	} testbuf;
+	unsigned int avg;
 	struct timespec my_time_spec_nano;
 
 	my_time_spec_nano.tv_sec = 0;
@@ -138,7 +148,8 @@ int probe_oss(const char *name_audio)
 		errprintf(SEV_WARNING, "audio driver inexact sampling rate: %d Hz (%d requested)\n", 
 			  apar, SAMPLE_RATE);
 	}
-        /*
+	
+	/*
          * set fragment sizes
          */
         apar = 0xffff0007U;
@@ -146,19 +157,39 @@ int probe_oss(const char *name_audio)
                 errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_SETFRAGMENT");
 	if (ioctl(fd_audio, SNDCTL_DSP_GETOSPACE, &oinfo) == -1)
 		errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_GETOSPACE");
+	osize = oinfo.fragstotal * oinfo.fragsize;
 	if (ioctl(fd_audio, SNDCTL_DSP_GETISPACE, &iinfo) == -1)
 		errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_GETISPACE");
-	if (iinfo.fragsize > pagesize)
-		errprintf(SEV_FATAL, "OSS: input: fragment size %d > page size %d\n",
+	isize = iinfo.fragstotal * iinfo.fragsize;
+	if (EXCESS_FRAGS * iinfo.fragsize > pagesize)
+		errprintf(SEV_FATAL, "OSS: input: fragment size %d times excess frags > page size %d\n",
 			  iinfo.fragsize, pagesize);
+	
         /*
          * determine if the driver is capable to support our access method
          */
         if (ioctl(fd_audio, SNDCTL_DSP_GETCAPS, &apar) == -1)
                 errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_GETCAPS");
-        if (!(apar & DSP_CAP_TRIGGER) || !(apar & DSP_CAP_MMAP))
-                errprintf(SEV_FATAL, "Sound driver does not support mmap and/or trigger\n");
+        if (!(apar & DSP_CAP_TRIGGER) || !(apar & DSP_CAP_MMAP)) {
+    	    errprintf(SEV_WARNING, "Sound driver does not support mmap and/or trigger\n");
+	    nommap = 1;	
+	}
+
+	/* a second mmap test,  as sometimes the first is false-positive */
+	
+	if ((testbuf.v = mmap(NULL, isize, PROT_READ, 
+			   MAP_ANON | MAP_PRIVATE, -1, 0)) == MAP_FAILED) {
+  		    /* sometimes mmap fails although the above test is o.k. */
+            errprintf(SEV_WARNING, "probe_oss: testbuf.v: mmap: MAP_ANONYMOUS failed.\n");
+	    nommap = 1;
+	    if (close(fd_audio))
+                errstr(SEV_FATAL, "close");
+	    }
+	if (!nommap)
+	    if (munmap(testbuf.v, isize))
+		errstr(SEV_FATAL, "fdx: munmap: testbuf.v: MAP_ANONYMOUS");
 	close(fd_audio);
+	
 #ifndef OSS_DISABLE_FDX
     if (apar & DSP_CAP_DUPLEX) {
 	errprintf(SEV_INFO, "Your soundcard is full duplex capable.\n");
@@ -169,7 +200,7 @@ int probe_oss(const char *name_audio)
 	}
 	else {
 	errprintf(SEV_INFO, 
-	    "Good; the reference time will be derived from the sample clock,\n"
+	    "Good; the reference time will be derived from the soundcard sample clock,\n"
 	    "which should be accurate within +-20ppm.\n"
 	    "So I hope your soundcard uses 'real' crystals instead of cheap ceramic\n"
 	    "resonators, and you adjusted the clock correctly with\n" 
@@ -219,15 +250,41 @@ int probe_oss(const char *name_audio)
                         errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_SETFRAGMENT");
                 if (ioctl(fd_audio, SNDCTL_DSP_GETISPACE, &iinfo) == -1)
                         errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_GETISPACE");
-                isize = iinfo.fragstotal * iinfo.fragsize;
+                 isize = iinfo.fragstotal * iinfo.fragsize;
                 if (ioctl(fd_audio, SNDCTL_DSP_GETCAPS, &apar) == -1)
                         errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_GETCAPS");
+			
+	        if (!(apar & DSP_CAP_TRIGGER) || !(apar & DSP_CAP_MMAP)) {
+        	    errprintf(SEV_WARNING, "Sound driver does not support mmap and/or trigger\n");
+		    nommap = 1;
+		    errprintf(SEV_WARNING, "So I can not calculate soundcard switch times for now.\n");
+
+	            if (close(fd_audio))
+                        errstr(SEV_FATAL, "close");
+	
+		    return 0;
+		}
+/*
+		original
                 if (!(apar & DSP_CAP_TRIGGER) || !(apar & DSP_CAP_MMAP))
                         errprintf(SEV_FATAL, "Sound driver does not support mmap and/or trigger");
+*/
+
 #ifdef FULL_MMAP
                 if ((ibuf = mmap(NULL, pagesize+isize, PROT_READ | PROT_WRITE, 
-				 MAP_ANON | MAP_PRIVATE, -1, 0)) == MAP_FAILED)
-                        errstr(SEV_FATAL, "probe: ibuf: mmap: MAP_ANONYMOUS");
+				 MAP_ANON | MAP_PRIVATE, -1, 0)) == MAP_FAILED){
+		    /* sometimes mmap fails although the above test is o.k. */
+		    
+           	    errprintf(SEV_WARNING, "probe_oss: ibuf: mmap: MAP_ANONYMOUS failed.\n");
+		    nommap = 1;
+		    errprintf(SEV_WARNING, "So I can not calculate soundcard switch times for now.\n");
+	            if (close(fd_audio))
+                        errstr(SEV_FATAL, "close");
+		    return 0;
+		}
+
+//			errstr(SEV_FATAL, "probe: ibuf: mmap: MAP_ANONYMOUS");
+			
                 ibufe = ibuf + isize + pagesize;
                 if (munmap(ibuf, isize))
                         errstr(SEV_FATAL, "probe: ibuf: munmap: MAP_ANONYMOUS");
@@ -237,7 +294,7 @@ int probe_oss(const char *name_audio)
 #else /* FULL_MMAP */
                 if ((ibuf = mmap(NULL, isize, PROT_READ, MAP_FILE | MAP_SHARED | MAP_FIXED,  
 				 fd_audio, 0)) == MAP_FAILED)
-                        errstr(SEV_FATAL, "probe: obuf: mmap: PROT_READ");
+                        errstr(SEV_FATAL, "probe: ibuf: mmap: PROT_READ");
 #endif /* FULL_MMAP */
                 apar = 0;
                 if (ioctl(fd_audio, SNDCTL_DSP_SETTRIGGER, &apar) == -1)
@@ -295,51 +352,19 @@ Guenther
 
 void l1_oss_fdx_mmap_cleanup(void *dummy) 
 {
-//	if (pthread_mutex_unlock(&l1mutex))
-//	    errstr(SEV_FATAL, "pthread_mutex_unlock");
-
-	int res = 0;
+	//int res = 0;
 	if (munmap(ibuf.v, isize))
 		errstr(SEV_FATAL, "fdx: munmap: ibuf: MAP_ANONYMOUS");
 	if (munmap(obuf.v, osize))
 		errstr(SEV_FATAL, "fdx: munmap: obuf: MAP_ANONYMOUS");
+	if (close(fd_audio))
+		errstr(SEV_FATAL, "close");
+	fd_audio = -1;
 
-	//printf("l1_oss_fdx_mmap_cleanup: before close: fd_audio is %d\n", 
-	//    fd_audio);
-	res = close (fd_audio);
-	if ( res < 0 ) {
-	//    printf ("negative result of close is %d\n", res);
-	    errstr(SEV_FATAL, 
-	    "l1_oss_fdx_mmap_cleanup: error in closing fd_audio: ");
-	} else {
-	//    printf ("o.k. result of close is %d\n", res);    
-	//    errprintf(SEV_INFO, 
-	//	"l1_oss_fdx_mmap_cleanup: audio device closed. res %d\n", res);
-	    fd_audio = -1;
-	}
-	//printf ("final check: result of close is %d\n", res);
-	//printf("l1_oss_fdx_mmap_cleanup: after close: fd_audio is %d\n", 
-	//    fd_audio);
-	//printf("l1_cleanup: fd_ptt is %d", fd_ptt);
-	/*
-	if (close (fd_ptt)) 
-	    errstr(SEV_INFO, "l1 thread cleanup: error in closing fd_ptt");
-	//else
-	//    errprintf(SEV_INFO, "l1 thread cleanup: ptt device closed\n");
-	//printf("l1_cleanup: fd_ptt is %d", fd_ptt);
-
-	//printf("l1_cleanup: fd_mixer is %d", fd_mixer);
-	if (close (fd_mixer)) 
-	    errstr(SEV_INFO, "l1 thread cleanup: error in closing fd_mixer");
-	//else
-	//    errprintf(SEV_INFO, "l1 thread cleanup: mixer device closed\n");
-	//printf("l1_cleanup: fd_mixer is %d", fd_mixer);
-	*/
 }
 
 /* --------------------------------------------------------------------- */
 
-#define EXCESS_FRAGS 3
 
 void *oss_fdx_driver(void *name_audio)
 {
@@ -357,6 +382,8 @@ void *oss_fdx_driver(void *name_audio)
 	short *s;
 	void *m = NULL;
 	
+	pthread_cleanup_push(l1_oss_fdx_mmap_cleanup, NULL);
+
         if ((fd_audio = open(name_audio, O_RDWR, 0)) < 0)
                 errstr(SEV_FATAL, "oss_fdx_driver: open");
 	//printf("oss_fdx_driver: fd_audio is %d\n", fd_audio);
@@ -448,7 +475,7 @@ void *oss_fdx_driver(void *name_audio)
 	 * loop
 	 */
 	for (;;) {
-		pthread_cleanup_push(l1_oss_fdx_mmap_cleanup, NULL);
+
 		if (pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL))
 		    errstr(SEV_FATAL, "pthread_setcancelstate");
 
@@ -459,12 +486,13 @@ void *oss_fdx_driver(void *name_audio)
 		i = select(fd_audio+1, &rmask, &wmask, NULL, NULL);
 		if (i < 0) 
 			errstr(SEV_FATAL, "select");
-		if (!FD_ISSET(fd_audio, &rmask) && !FD_ISSET(fd_audio, &wmask))
-			continue;
 		
 		if (pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL))
 		    errstr(SEV_FATAL, "pthread_setcancelstate");
-		pthread_cleanup_pop(0);	
+		
+		if (!FD_ISSET(fd_audio, &rmask) && !FD_ISSET(fd_audio, &wmask))
+			continue;
+		
 		/*
 		 * process input
 		 */
@@ -563,21 +591,30 @@ void *oss_fdx_driver(void *name_audio)
 		}
 		output_ptt(ptt_frames > 0);
 	}
+	pthread_cleanup_pop(0);	
 }
 
+/* --------------------------------------------------------------------- */
+
+void l1_oss_hdx_mmap_cleanup(void *dummy) 
+{
+	if (munmap(buf.v, size))
+		errstr(SEV_FATAL, "munmap");
+	if (close(fd_audio))
+		errstr(SEV_FATAL, "close");
+	fd_audio = -1;
+
+}
+
+
 /* --------------------------------------------------------------------- */
 
 void *oss_hdx_driver(void *name_audio)
 {
         size_t pagesize = getpagesize();
         int apar, i;
-	union {
-		caddr_t v;
-		short *s;
-	} buf;
 	caddr_t buf_ext;
 	struct audio_buf_info info;
-	unsigned int size;
 	unsigned int fragptr;
 	l1_time_t inc_sample;
 	l1_time_t inc_fragment;
@@ -590,7 +627,11 @@ void *oss_hdx_driver(void *name_audio)
 	short *s;
 	void *m = NULL;
 
+	pthread_cleanup_push(l1_oss_hdx_mmap_cleanup, NULL);
+
 	refclock_init();
+
+
 	for (;;) {
 		/*
 		 * start receiver
@@ -674,16 +715,28 @@ void *oss_hdx_driver(void *name_audio)
 		 * loop
 		 */
 		for (;;) {
+	
+
+			if (pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL))
+			    errstr(SEV_FATAL, "hdx: pthread_setcancelstate");
+
+	
 			FD_ZERO(&mask);
 			FD_SET(fd_audio, &mask);
 			i = select(fd_audio+1, &mask, NULL, NULL, NULL);
 			if (i < 0) 
 				errstr(SEV_FATAL, "select");
+
+			if (pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL))
+			    errstr(SEV_FATAL, "hdx: pthread_setcancelstate");
+
 			if (!FD_ISSET(fd_audio, &mask))
 				continue;
+
 			/*
 			 * process input
 			 */
+
 			if (ioctl(fd_audio, SNDCTL_DSP_GETIPTR, &cinfo) == -1)
 				errstr(SEV_FATAL, "ioctl: SNDCTL_DSP_GETIPTR");
 			l1time = refclock_current(((size + cinfo.ptr - last_dmaptr) % size) / 2 * inc_sample, 1);
@@ -716,10 +769,16 @@ void *oss_hdx_driver(void *name_audio)
 					fragptr = 0;
 			}
 			/*
-			 * check for output
+			 * check for fsk output
 			 */
 			if (l1_fsk_next_tx_event(l1time) <= (long)inc_fragment/2)
 				break;
+			/*
+			 * check for mt63 output
+			 */
+			if (mt63_data_waiting)
+				break;
+
 		}
 		if (munmap(buf.v, size))
 			errstr(SEV_FATAL, "munmap");
@@ -812,11 +871,20 @@ void *oss_hdx_driver(void *name_audio)
 		 * loop
 		 */
 		for (;;) {
+		
+			if (pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL))
+			    errstr(SEV_FATAL, "hdx: pthread_setcancelstate");
+
 			FD_ZERO(&mask);
 			FD_SET(fd_audio, &mask);
 			i = select(fd_audio+1, NULL, &mask, NULL, NULL);
 			if (i < 0) 
 				errstr(SEV_FATAL, "select");
+			
+			if (pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL))
+			    errstr(SEV_FATAL, "hdx: pthread_setcancelstate");
+
+			
 			if (!FD_ISSET(fd_audio, &mask))
 				continue;
 			/*
@@ -856,50 +924,19 @@ void *oss_hdx_driver(void *name_audio)
 			errstr(SEV_FATAL, "munmap");
 		if (close(fd_audio))
 			errstr(SEV_FATAL, "close");
+		
 	}
+	pthread_cleanup_pop(0);	
 }
 
 /* --------------------------------------------------------------------- */
 
 void l1_oss_fdx_nommap_cleanup(void *dummy) 
 {
-//	if (pthread_mutex_unlock(&l1mutex))
-//	    errstr(SEV_FATAL, "pthread_mutex_unlock");
-
-	int res = 0;
-	//printf("l1_oss_fdx_mmap_cleanup: before close: fd_audio is %d\n", 
-	//    fd_audio);
-	printf("l1_oss_fdx_nommap_cleanup: before close\n");
-	res = close (fd_audio);
-	if ( res < 0 ) {
-	//    printf ("negative result of close is %d\n", res);
+	if ( close (fd_audio))
 	    errstr(SEV_FATAL, 
 	    "l1_oss_fdx_nommap_cleanup: error in closing fd_audio: ");
-	} else {
-	//    printf ("o.k. result of close is %d\n", res);    
-	//    errprintf(SEV_INFO, 
-	//	"l1_oss_fdx_mmap_cleanup: audio device closed. res %d\n", res);
-	    fd_audio = -1;
-	}
-	printf("l1_oss_fdx_nommap_cleanup: after close\n");
-	//printf ("final check: result of close is %d\n", res);
-	//printf("l1_oss_fdx_mmap_cleanup: after close: fd_audio is %d\n", 
-	//    fd_audio);
-	//printf("l1_cleanup: fd_ptt is %d", fd_ptt);
-	/*
-	if (close (fd_ptt)) 
-	    errstr(SEV_INFO, "l1 thread cleanup: error in closing fd_ptt");
-	//else
-	//    errprintf(SEV_INFO, "l1 thread cleanup: ptt device closed\n");
-	//printf("l1_cleanup: fd_ptt is %d", fd_ptt);
-
-	//printf("l1_cleanup: fd_mixer is %d", fd_mixer);
-	if (close (fd_mixer)) 
-	    errstr(SEV_INFO, "l1 thread cleanup: error in closing fd_mixer");
-	//else
-	//    errprintf(SEV_INFO, "l1 thread cleanup: mixer device closed\n");
-	//printf("l1_cleanup: fd_mixer is %d", fd_mixer);
-	*/
+        fd_audio = -1;
 }
 
 /* --------------------------------------------------------------------- */
