@@ -42,13 +42,20 @@
 #include <signal.h>
 #include <syslog.h>
 #include <sys/types.h>
+#ifdef __linux__
 #include <asm/byteorder.h>
+#endif
 #include <sys/soundcard.h>
 #include <sys/socket.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
+#ifdef __linux__
 #include <endian.h>
+#endif
+#ifdef __FreeBSD__ 
+#include <sys/endian.h>
+#endif
 #include <assert.h>
 
 #ifdef DISABLE_REALTIME
@@ -75,21 +82,22 @@
 
 /* these variables take hfkernel's options */
 static int logging = 0;
-const char *name_audio = NULL;
+const char *name_audio ="/dev/dsp";
+const char *name_audio_OSS ="/dev/dsp";
 const char *name_ptt = NULL;
 const char *name_mixer = NULL;
 float snd_corr = 1, tv_corr = 1, cpu_mhz = 0;
 int dis_rdtsc = 0, invert_ptt = 0;
 const char *name_comm = "/var/run/hfapp";
+int OSS = 0, ALSA = 0, fdx = 0;
 int force_halfduplex = 0;
 int nommap = 0;
+int fsk_l1_loop_running = 0;
 unsigned int standby_flags = 
   STANDBY_FLAG_FREQ_ESTIMATION|STANDBY_FLAG_MONITOR_200|STANDBY_FLAG_MONITOR_300;
 unsigned int comm_perm = S_IRWXU|S_IRWXG|S_IRWXO;
-
 extern int brake, elbug_rts;
 pthread_t thr_l2;
-
 int fd_audio = -1, fd_mixer = -1, fd_ptt = -1;
 pthread_t thr_l1;
 int samples_remain = 0;
@@ -146,6 +154,26 @@ void output_ptt(int ptt)
 	}
 }
 
+
+/* --------------------------------------------------------------------- */
+
+/* to shut down l1 fsk soundcard loop  */
+void l1_stop()
+{
+	if (! fsk_l1_loop_running) return;	
+
+	if (pthread_cancel(thr_l1))
+		errstr(SEV_WARNING, "pthread_cancel l1 thread");
+	if (pthread_join(thr_l1, NULL))
+		errstr(SEV_WARNING, "pthread_join l1 thread");
+	fsk_l1_loop_running = 0;	
+	printf ("l1 fsk soundcard loop is finished.\n");
+	bufprintf(HFAPP_MSG_DATA_STATUS, "l1 fsk soundcard loop is finished.");
+//	sleep(1);
+	return;
+}
+
+
 /* --------------------------------------------------------------------- */
 
 /* thought i need it for mt63, but no need now, maybe later */
@@ -225,46 +253,18 @@ void l1_switch_to_mmap()
 
 void l1_init()
 {
+	
+//	test & start low-level soundcard loop, for fsk modes only
 	static struct sched_param schp;
 	pthread_attr_t thr_attr;
-	int OSS = 0, ALSA = 0, fdx = 0;
-
-	if (!name_audio) {
-	    name_audio = "/dev/dsp";
-	    printf("hfkernel: l1_init: will try OSS driver for %s\n", 
-		name_audio);
-	    OSS = 1;
-	} else if (strstr (name_audio, "dev/")) {
-	    printf("hfkernel: l1_init: will try OSS driver for %s\n", 
-		name_audio);
-	    OSS = 1;	
-	} else if (strstr (name_audio, "hw:")) {
-#ifndef HAVE_ALSA_ASOUNDLIB_H
-	    errprintf(SEV_FATAL, "hfkernel: l1_init: Soundcard name \n"
-		"contains 'hw:...', which is ALSA convention,  \n"
-		"but program is not compiled with ALSA driver.\n");
-	    exit (1);
-#endif /* HAVE_ALSA_ASOUNDLIB_H */ 
-	    printf("hfkernel: l1_init: will try ALSA driver for '...hw:...'\n");
-	    ALSA = 1;	
-	} else {
-	    errprintf(SEV_FATAL, "hfkernel: l1_init: Soundcard name \n"
-		"does not contain 'dev/...' (OSS) and not 'hw:...'(ALSA) -> \n"
-		"please check soundcard option in /etc/hf.conf\n"
-		"because I want to know what driver to use....\n");
-	    exit (1);
-	}
-	if (OSS) {
-	    fdx = probe_oss(name_audio);
-	}
-	l1_fsk_modem_init();
-	if (name_ptt) {
-	    if ((fd_ptt = open(name_ptt, O_RDWR, 0)) < 0)
-		errstr(SEV_WARNING, "open ptt device");
-	} else
-	    fd_ptt = -1;
-	output_ptt(0);
 	
+	if (fsk_l1_loop_running) return;
+	
+	//fsk_l1_loop_running = 1;	
+	// again for sure
+	//if (close(fd_audio)) errstr(SEV_FATAL, "l1_init: audio device could not be closed.");
+
+	l1_fsk_modem_init();
 	if (OSS) {
 	    errprintf(SEV_INFO, "preparing mixer ...\n");
 	    oss_prepare_mixer(name_mixer);
@@ -291,6 +291,7 @@ void l1_init()
 			errstr(SEV_FATAL, "pthread_create");
 		    if (pthread_attr_destroy(&thr_attr))
 			errstr(SEV_WARNING, "pthread_attr_destroy");
+		    fsk_l1_loop_running = 1;	
 		    return;
 		}  else {
 		    if (pthread_create(&thr_l1, &thr_attr, 
@@ -298,6 +299,7 @@ void l1_init()
 			errstr(SEV_FATAL, "pthread_create");
 		    if (pthread_attr_destroy(&thr_attr))
 			errstr(SEV_WARNING, "pthread_attr_destroy");
+		    fsk_l1_loop_running = 1;	
 		    return;
 		}
 	    }
@@ -311,6 +313,7 @@ void l1_init()
 		errstr(SEV_FATAL, "pthread_create");
 	    if (pthread_attr_destroy(&thr_attr))
 		errstr(SEV_WARNING, "pthread_attr_destroy");
+	    fsk_l1_loop_running = 1;	
 	    return;
 	}
 #endif /* HAVE_ALSA_ASOUNDLIB_H */
@@ -326,6 +329,7 @@ void l1_init()
 	    	    errstr(SEV_FATAL, "pthread_create");	
 		if (pthread_attr_destroy(&thr_attr))
 		    errstr(SEV_WARNING, "pthread_attr_destroy");
+		fsk_l1_loop_running = 1;	
 		return;
 	    } else {
 		if (pthread_create(&thr_l1, &thr_attr, oss_hdx_driver, 
@@ -333,6 +337,7 @@ void l1_init()
 	    	    errstr(SEV_FATAL, "pthread_create");	
 		if (pthread_attr_destroy(&thr_attr))
 		    errstr(SEV_WARNING, "pthread_attr_destroy");
+		fsk_l1_loop_running = 1;	
 		return;
 	    }
 	}
@@ -344,7 +349,8 @@ void l1_input_samples(l1_time_t tstart, l1_time_t tinc,
 		      short *samples, unsigned int nsamples)
 {
 	if (modefamily == MT63) {
-	    l1_mt63_input_samples(samples, nsamples);
+	    //l1_mt63_input_samples(samples, nsamples);
+	    errprintf(SEV_WARNING, "l1_input_samples should not have been called.\n");
 	    return;
 	}
 	else if (modefamily == FSK) {
@@ -363,12 +369,14 @@ int l1_output_samples(l1_time_t tstart, l1_time_t tinc,
 		      short *samples, unsigned int nsamples)
 {
 	int retval; 
-	if (modefamily == MT63) {
-	    retval = l1_mt63_output_samples(tstart, tinc, samples, nsamples);
+	if (modefamily == FSK) {
+	    retval = l1_fsk_output_samples(tstart, tinc, samples, nsamples);
 	    return retval;
 	}
-	else if (modefamily == FSK) {
-	    retval = l1_fsk_output_samples(tstart, tinc, samples, nsamples);
+	else if (modefamily == MT63) 
+	{
+	    //retval = l1_mt63_output_samples(tstart, tinc, samples, nsamples);
+	    errprintf(SEV_WARNING, "l1_output_samples should not have been called.\n");
 	    return retval;
 	}
 	else 
@@ -420,10 +428,24 @@ static void start_threads(void)
 	if (pthread_attr_setschedparam(&thr_attr, &schp))
 	    errstr(SEV_WARNING, "pthread_attr_setschedparam");
 #endif
-/* here a primitive test for implementing new modes... */
+
+/* here a primitive test for implementing new modes... 
+ * for this hfkernel needs to be started only in console mode 
+ * We must declare the modefamily first: MT63 od CW od FSK.
+ * rtty, standby, amtor, pactor, gtor are the fsk modes.
+ * If you test a fsk, l1_init() first.
+ * 
+ */
+	
+//	modefamily = MT63;
+	modefamily = FSK;
+//	modefamily = 0;
+	l1_init();
 //	if (pthread_create(&thr_l2, &thr_attr, mode_cw_tx, NULL))
 	if (pthread_create(&thr_l2, &thr_attr, mode_standby, NULL))
+//	if (pthread_create(&thr_l2, &thr_attr, mode_mt63_tx, NULL))
 	    errstr(SEV_FATAL, "pthread_create");
+
 #ifdef HAVE_SCHED_H
 	schp.sched_priority = sched_get_priority_min(SCHED_RR);
 	if (pthread_attr_setschedparam(&thr_attr, &schp))
@@ -441,6 +463,37 @@ static void start_threads(void)
 
 /* --------------------------------------------------------------------- */
 
+static void start_io_thread(void)
+{
+#ifdef HAVE_SCHED_H
+	struct sched_param schp;
+#endif
+	pthread_attr_t thr_attr;
+	pthread_t thr_io;
+     
+	signal(SIGPIPE, SIG_IGN);
+	if (pthread_attr_init(&thr_attr))
+	    errstr(SEV_FATAL, "pthread_attr_init");
+#ifdef HAVE_SCHED_H
+	memset(&schp, 0, sizeof(schp));
+	if (pthread_attr_setschedpolicy(&thr_attr, SCHED_RR))
+	    errstr(SEV_WARNING, "pthread_attr_setschedpolicy");
+	schp.sched_priority = sched_get_priority_min(SCHED_RR);
+	if (pthread_attr_setschedparam(&thr_attr, &schp))
+	    errstr(SEV_WARNING, "pthread_attr_setschedparam");
+#endif
+	if (pthread_create(&thr_io, &thr_attr, io_process, NULL))
+	    errstr(SEV_FATAL, "pthread_create");
+	if (pthread_attr_destroy(&thr_attr))
+	    errstr(SEV_WARNING, "pthread_attr_destroy");
+	if (pthread_join(thr_io, NULL))
+	    errstr(SEV_WARNING, "pthread_join");
+	if (munlockall())
+	    errstr(SEV_WARNING, "mlockall");
+}
+
+/* --------------------------------------------------------------------- */
+
 int main(int argc, char *argv[])
 {
         int c;
@@ -484,12 +537,16 @@ int main(int argc, char *argv[])
 
                 case 'm':
 		    pre_cpumhz = strtod(optarg, NULL);
+		    
+/*
 		    if (pre_cpumhz < 10 || pre_cpumhz > 16000) {
 			fprintf(stderr, 
 			    "CPU clock out of range 10MHz..16000MHz\n");
 			err++;
 		    }
-		    cpu_mhz = pre_cpumhz;
+*/
+		    cpu_mhz = pre_cpumhz; 
+
 		    break;
 
                 case 'n':
@@ -561,7 +618,7 @@ int main(int argc, char *argv[])
 			"  -3: standby: disable monitoring of 300baud signals\n"
 			"  -a: audio device path (default for OSS: /dev/dsp)\n"
 			"      (for ALSA driver use -a plughw:0,0)\n"
-			"  -c: path of the communication socket (default: hfapp)\n"
+			"  -c: path of the communication socket (default: /var/run/hfapp)\n"
 			"  -f: standby: disable frequency estimation\n"			
 			"  -h: force half duplex mode (for OSS only!)\n"			
 			"  -i: invert PTT (default: PTT = positive signal)\n"
@@ -580,14 +637,68 @@ int main(int argc, char *argv[])
         }
         if (logging)
                 openlog("hfkernel", LOG_PID, LOG_DAEMON);
-	errprintf(SEV_INFO, "hfkernel %s starting...\n", VERSION);
+	printf("hfkernel %s starting...\n", PACKAGE_VERSION);
 #ifndef DISABLE_REALTIME
 	if (mlockall(MCL_CURRENT))
 		errstr(SEV_WARNING, "mlockall");
 #endif
-	l1_init();
- 	errno = EAGAIN;
-	start_threads();
-	exit(0);
-}
+	errno = EAGAIN;
+
+	if (!name_audio) {
+	    name_audio = "/dev/dsp";
+	    printf("hfkernel: I will try OSS driver for %s\n", 
+		name_audio);
+	    OSS = 1;
+	} else if (strstr (name_audio, "dev/")) {
+	    printf("hfkernel: I will try OSS driver for %s\n", 
+		name_audio);
+	    OSS = 1;	
+	} else if (strstr (name_audio, "hw:")) {
+#ifndef HAVE_ALSA_ASOUNDLIB_H
+	    errprintf(SEV_FATAL, "hfkernel: Soundcard name contains 'hw:...', "
+		"which is ALSA convention, but I am not compiled with ALSA driver here.\n");
+	    exit (1);
+#endif /* HAVE_ALSA_ASOUNDLIB_H */ 
+	    ALSA = 1;	
+	    if (strstr (name_audio, "hw:0")) 
+		name_audio_OSS="/dev/dsp0";
+	    if (strstr (name_audio, "hw:1")) 
+		name_audio_OSS="/dev/dsp1";
+	    if (strstr (name_audio, "hw:2")) 
+		name_audio_OSS="/dev/dsp2";
+	    if (strstr (name_audio, "hw:3")) 
+		name_audio_OSS="/dev/dsp3";
+            printf("For FSK modes, I will try ALSA code for device %s\n"
+		"but for MT63, I will use OSS code for device %s\n", name_audio, name_audio_OSS);
+	} else {
+	    errprintf(SEV_FATAL, 
+		"Soundcard name does not contain 'dev/...' (OSS) and not 'hw:...'(ALSA) -> \n"
+		"please check soundcard option, because I want to know what driver to use....\n");
+	    exit (1);
+ 	}
+
+	if (OSS) {
+	    fdx = probe_oss(name_audio);
+	}
+
+	if (name_ptt) {
+	    if ((fd_ptt = open(name_ptt, O_RDWR, 0)) < 0)
+		errstr(SEV_WARNING, "error at opening ptt device\n");
+	    else {
+	        errprintf(SEV_INFO, "opened ptt device with fd %d\n", fd_ptt);
+		output_ptt(0);
+	    } 
+	 } else 
+	    fd_ptt = -1;
+	
+	/* this is for testing new modes only.#
+	 * From 0.7.4. on the mode-threads are controlled from hfterm 
+	 */
+	
+//	start_threads();
+
+	printf("Note: hfkernel is only part of the hf package.\n"); 
+	printf("It is controlled by the graphic terminal hfterm. To start them both, use the start script hf. In newer linuxes (kernel 2.6...) we need the syntax\n ´LD_ASDSUME_KERNEL=2.2.5 hfterm´, this is already prepared in the hf script. \n");
+	start_io_thread();
+	exit(0); }
 
