@@ -60,7 +60,8 @@
 /*
  * Buffering
  */
-static unsigned char txbuf[16384];
+//static unsigned char txbuf[16384];
+static unsigned char txbuf[32768];
 static unsigned int txb_rd = 0, txb_wr = 0;
 static pthread_mutex_t txb_mutex = PTHREAD_MUTEX_INITIALIZER;
 
@@ -93,7 +94,7 @@ static struct {
 int modefamily;
 #define FSK 1
 #define MT63 2
-
+#define CW 3
 int beaconpause = 5;
 
 /* --------------------------------------------------------------------- */
@@ -287,32 +288,105 @@ void bufprintf(unsigned int which, const char *fmt, ...)
 	msg.hdr.len = htonl(i);
 	msg.hdr.err = ERR_NOERR;
 	send_msg(&msg);
+//	debug only, prints lot of mess...
+//	printf("%s", fmt);
 }
 
 /* --------------------------------------------------------------------- */
 
 /* compiler warn: "static decl follows non-static" */
-/*static*/ void start_l2_thread(void *(*routine)(void *))
+/*static*/ 
+//modi
+void start_l2_thread(void *(*routine)(void *))
 {
 #ifndef NOREALTIME
 	static struct sched_param schp;
 #endif /* NOREALTIME */
 	pthread_attr_t thr_attr;
 	static int crashblock = 0;
+	int res = 0;
+	
+//	printf("start_l2_thread: modefamily is %d\n", modefamily);
 
-	if(crashblock > 0) {
+	if (pthread_attr_init(&thr_attr))
+		errstr(SEV_FATAL, "pthread_attr_init");
+
+	if (crashblock) {
 	    bufprintf(HFAPP_MSG_DATA_STATUS, 
 		"start of new mode in progress.\n"
-		"-> no mode-changing commands accepted for now.");
+		"-> no mode-changing commands accepted for now.\n");
 	    return;
 	}
 	crashblock = 1;
 	
+	res =(pthread_cancel(thr_l2));
+	//res =(pthread_kill(thr_l2, NULL));
+	//kill does not work
+	
+	if (res){
+	    if (res == ESRCH) {
+	    	errstr(SEV_INFO, "pthread_cancel: no l2 thread was running");
+		res = 0;
+	    } else {
+		errstr(SEV_WARNING, "pthread_cancel l2 thread: unknown error");
+	    }
+	}
+	
+	if (!res) {
+	    if (pthread_join(thr_l2, NULL))
+		errstr(SEV_WARNING, "pthread_join l2 thread");
+	}
+	
+	// switch fsk sound loop on / off 
+	
+	if ((fsk_l1_loop_running) && (modefamily == FSK)) {
+//	    printf("mode family remains FSK, so FSK sound loop keeps running...\n");
+	}
+	
+	if ((! fsk_l1_loop_running) && (modefamily == FSK)) {
+	    l1_init();
+//	    printf("mode family changed to FSK, I started l1 FSK sound loop.\n");
+	}
+	
+	if ( ( fsk_l1_loop_running) && (modefamily != FSK)) {
+	    l1_stop();
+//	    printf("mode family changed, I stopped l1 FSK sound loop.\n");
+	}
+	if ( (! fsk_l1_loop_running) && (modefamily != FSK)) {
+//	    printf("mode family remains other than FSK.\n");
+	}
+
+#ifndef NOREALTIME
+ 	memset(&schp, 0, sizeof(schp));
+	if (pthread_attr_setschedpolicy(&thr_attr, SCHED_RR))
+		errstr(SEV_FATAL, "pthread_attr_setschedpolicy");
+	schp.sched_priority = sched_get_priority_min(SCHED_RR)+1;
+	if (pthread_attr_setschedparam(&thr_attr, &schp))
+		errstr(SEV_FATAL, "pthread_attr_setschedparam");
+#endif /* NOREALTIME */
+        
+	if (pthread_create(&thr_l2, &thr_attr, routine, NULL))
+		errstr(SEV_FATAL, "pthread_create");
+	
+	if (pthread_attr_destroy(&thr_attr))
+		errstr(SEV_WARNING, "pthread_attr_destroy");
+	
+	crashblock = 0;
+
+}
+
+// Tom's original
+static void ori_start_l2_thread(void *(*routine)(void *))
+{
+#ifndef NOREALTIME
+	static struct sched_param schp;
+#endif /* NOREALTIME */
+	pthread_attr_t thr_attr;
+
 	if (pthread_cancel(thr_l2))
 		errstr(SEV_FATAL, "pthread_cancel");
 	if (pthread_join(thr_l2, NULL))
 		errstr(SEV_FATAL, "pthread_join");
-	modefamily = 0;
 	if (pthread_attr_init(&thr_attr))
 		errstr(SEV_FATAL, "pthread_attr_init");
 #ifndef NOREALTIME
@@ -323,19 +397,16 @@ void bufprintf(unsigned int which, const char *fmt, ...)
 	if (pthread_attr_setschedparam(&thr_attr, &schp))
 		errstr(SEV_FATAL, "pthread_attr_setschedparam");
 #endif /* NOREALTIME */
-        /* for sure, once more ... */
-        l1_fsk_clear_requests();
-	/* because cw elbug changes space to 0 and mark to cwtone... */
-	l1_set_fsk_params(FREQ_SPACE, FREQ_MARK);
 	if (pthread_create(&thr_l2, &thr_attr, routine, NULL))
 		errstr(SEV_FATAL, "pthread_create");
 	if (pthread_attr_destroy(&thr_attr))
 		errstr(SEV_WARNING, "pthread_attr_destroy");
-	crashblock = 0;
 }
 
 /* --------------------------------------------------------------------- */
 
+/* --------------------------------------------------------------------- */
+
 static void process_msg(struct hfapp_msg *msg)
 {
 	unsigned int type = ntohl(msg->hdr.type);
@@ -356,6 +427,8 @@ static void process_msg(struct hfapp_msg *msg)
 		if (len < sizeof(msg->data.u))
 			return;
 		u = ntohl(msg->data.u);
+		//printf("hfkernel: sample request for spectrum by hfterm.\n");
+
 		if (u >= sizeof(samples_buf.buf)/sizeof(samples_buf.buf[0])) {
 			samples_buf.hdr.type = htonl(HFAPP_MSG_ACK_SAMPLES);
 			samples_buf.hdr.len = htonl(0);
@@ -388,8 +461,6 @@ static void process_msg(struct hfapp_msg *msg)
 			kbd_put(((*bp) & KBD_CHAR) | KBD_BIN);
 		return;
 
-
-
 	case HFAPP_MSG_SET_CWPAR:
 		if (len < sizeof(msg->data.cwpar))
 			return;
@@ -401,10 +472,12 @@ static void process_msg(struct hfapp_msg *msg)
 		return;
 
 	case HFAPP_MSG_MODE_CW_RX:
+		modefamily = CW;
 //		start_l2_thread(mode_cw_rx);
 		return;	
 		
 	case HFAPP_MSG_MODE_CW_TX:
+		modefamily = CW;
 		start_l2_thread(mode_cw_tx);
 		return;	
 		
@@ -425,18 +498,22 @@ static void process_msg(struct hfapp_msg *msg)
 		return;
 
 	case HFAPP_MSG_START_STANDBY:
+		modefamily = FSK;
 		start_l2_thread(mode_standby);
 		return;
 
 	case HFAPP_MSG_START_PACTOR_STANDBY:
+		modefamily = FSK;
 		start_l2_thread(mode_pactor_standby);
 		return;
 
 	case HFAPP_MSG_START_PACTOR_ARQ:
+		modefamily = FSK;
 		start_l2_thread(mode_pactor_arq);
 		return;
 
 	case HFAPP_MSG_START_PACTOR_FEQ:
+		modefamily = FSK;
 		start_l2_thread(mode_pactor_fec);
 		return;
 
@@ -455,6 +532,7 @@ static void process_msg(struct hfapp_msg *msg)
 		return;
 
 	case HFAPP_MSG_START_GTOR_ARQ:
+		modefamily = FSK;
 		start_l2_thread(mode_gtor_arq);
 		return;
 
@@ -494,14 +572,17 @@ static void process_msg(struct hfapp_msg *msg)
 		return;
 
 	case HFAPP_MSG_START_AMTOR_ARQ:
+		modefamily = FSK;
 		start_l2_thread(mode_amtor_arq);
 		return;
 
 	case HFAPP_MSG_START_AMTOR_COLFEQ:
+		modefamily = FSK;
 		start_l2_thread(mode_amtor_colfec);
 		return;
 
 	case HFAPP_MSG_START_AMTOR_SELFEQ:
+		modefamily = FSK;
 		start_l2_thread(mode_amtor_selfec);
 		return;
 
@@ -518,17 +599,20 @@ static void process_msg(struct hfapp_msg *msg)
 		return;
 
 	case HFAPP_MSG_START_RTTY_TX:
+		modefamily = FSK;
 		start_l2_thread(mode_rtty_tx);
 		return;
 
 	case HFAPP_MSG_START_RTTY_RX:
+		modefamily = FSK;
 		start_l2_thread(mode_rtty_rx);
 		return;
 
 	case HFAPP_MSG_SET_RTTYPAR:
 		if (len < sizeof(msg->data.rpar))
 			return;
-		rtty_set_params(htons(msg->data.rpar.baud), 
+//		rtty_set_params(htons(msg->data.rpar.baud), 
+		rtty_set_params (msg->data.rpar.baud, 
 		    msg->data.rpar.rxinvert, 
 		    msg->data.rpar.txinvert);
 		return;
@@ -566,40 +650,12 @@ static void process_msg(struct hfapp_msg *msg)
 		return;
 		
 	case HFAPP_MSG_START_MT63_TX:
-		start_l2_thread(mode_mt63_tx);
-		return;
-
-	case HFAPP_MSG_START_MT63_500_TX:
-		mt63p.bandwidth = 500;
-		start_l2_thread(mode_mt63_tx);
-		return;
-
-	case HFAPP_MSG_START_MT63_1000_TX:
-		mt63p.bandwidth = 1000;
-		start_l2_thread(mode_mt63_tx);
-		return;
-
-	case HFAPP_MSG_START_MT63_2000_TX:
-		mt63p.bandwidth = 2000;
+		modefamily = MT63;
 		start_l2_thread(mode_mt63_tx);
 		return;
 
 	case HFAPP_MSG_START_MT63_RX:
-		start_l2_thread(mode_mt63_rx);
-		return;
-
-	case HFAPP_MSG_START_MT63_500_RX:
-		mt63p.bandwidth = 500;
-		start_l2_thread(mode_mt63_rx);
-		return;
-
-	case HFAPP_MSG_START_MT63_1000_RX:
-		mt63p.bandwidth = 1000;
-		start_l2_thread(mode_mt63_rx);
-		return;
-
-	case HFAPP_MSG_START_MT63_2000_RX:
-		mt63p.bandwidth = 2000;
+		modefamily = MT63;
 		start_l2_thread(mode_mt63_rx);
 		return;
 
@@ -746,7 +802,7 @@ void *io_process(void *dummy)
 		/*
 		 * do periodic checks
 		 */
-		if (l1_sample_finished()) {
+		if (l1_sample_finished()) { 
 			for (i = ntohl(samples_buf.hdr.len)/2-1; i >= 0; i--)
 				samples_buf.buf[i] = htons(samples_buf.buf[i]);
 			send_msg((struct hfapp_msg *)&samples_buf);
