@@ -104,7 +104,8 @@ struct hfapp_msg {
 		} cwpar;
 
 		struct hfapp_msg_rtty_params {
-			u_int16_t baud;
+//			u_int16_t baud;
+			float baud;
 			unsigned char rxinvert;
 			unsigned char txinvert;
 		} rpar;
