@@ -34,6 +34,9 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
+#ifdef __FreeBSD__ 
+#include <unistd.h>
+#endif
 
 #include "dsp.h"
 #include "mt63.h"
@@ -115,7 +118,8 @@ int mt63_rx_start(unsigned int bandwidth, unsigned int doubleinterleave,
   }
   UseRateConv = (DevRate != 8000.0);
   if(UseRateConv) { 
-    err=RateConv.Preset(16,NULL,64);
+//    err=RateConv.Preset(16,NULL,64);
+    err=RateConv.Preset(16,(float**)NULL,64);
       if(err) { 
         printf
           ("Can't preset the rate converter - not enough RAM ?!\n"); 
@@ -166,7 +170,7 @@ int mt63_tx_start(unsigned int bandwidth, unsigned int doubleinterleave,
     case 2000:
 	break;
     default: 
-	printf("mt63_tx: invalid bandwidth option %d\n",
+	printf("mt63_tx_start: invalid bandwidth option %d\n",
 	bandwidth);
 	exit (1);
   }
@@ -178,7 +182,7 @@ int mt63_tx_start(unsigned int bandwidth, unsigned int doubleinterleave,
   }
   UseRateConv = (DevRate != 8000.0);
   if(UseRateConv) { 
-    err=RateConv.Preset(16,NULL,64);
+    err=RateConv.Preset(16,(float**)NULL,64);
       if(err) { 
         printf
           ("Can't preset the rate converter - not enough RAM ?!\n"); 
@@ -220,16 +224,27 @@ Error:
 int open_audio_input(char *DevName) {
   int err;
 
-  printf("Audio device for input is %s\n", DevName);
-  err = RxDev.OpenForRead(DevName,(int)UserRate);
-  if(err<0) { 
-      printf("Can't open %s to read audio from: errno => %s\n",
-	DevName,strerror(errno)); 
-	return (1); 
+    printf("Audio device for input is %s\n", DevName);
+     err = RxDev.OpenForRead(DevName,(int)UserRate);
+     if(err<0) { 
+	  printf("Can't open %s to read audio from: errno => %s\n",
+	    DevName,strerror(errno)); 
+	exit(1); 
+      }
+    printf("Audio device opened for input: DevNo = %d\n", err);
+    
+// for test only
+/*
+  { err=RxDev.OpenFileForRead("/home/monday/mtf",8000 ); //(int)UserRate);
+    if(err<0)
+    { printf("Can't open %s to read audio from file\n"); exit (1); }
+    printf("We read the audio from /home/monday/mtf\n");
   }
+ */
   return(0);
 }
 
+
 // ============================================================================
 
 int open_audio_output(char *DevName) {
@@ -240,8 +255,9 @@ int open_audio_output(char *DevName) {
   if(err<0) { 
       printf("Can't open %s to write audio to: errno => %s\n",
 	DevName,strerror(errno)); 
-	return (1); 
+	exit(1); 
   }
+  printf("Audio device opened for output: DevNo = %d\n", err);
   return(0);
 }
 
@@ -250,6 +266,7 @@ int open_audio_output(char *DevName) {
 void mt63_decode(short *samples, int count) 
 { int i, InpLen; 
     memset(code, 0, sizeof(code));
+    
   //printf("decoding mt63:\n");
   for(i = 0; i < count && i < BuffLen; i++) {
     InpBuff[i] = (s16)samples[i];
@@ -339,6 +356,69 @@ void mt63_direct_rx()
 
 // ============================================================================
 
+
+void mt63_rx() 
+{ 
+  int i, k, err, InpLen; 
+  
+    InpLen=RxDev.Read(InpBuff,BuffLen);
+    if(InpLen<0){ 
+	printf("Error while reading audio from audio, errno => %s\n",
+        strerror(errno)); exit(1); 
+    }
+    if(InpLen==0) { 
+	usleep(50000); 
+	printf("wait...\n");
+    }
+    
+    // write samples to the buffer for the spectrum
+    if (samples_remain > 0) {
+    	k = 2*samples_remain;
+	if (k > BuffLen)
+	    k = BuffLen;
+	memcpy(samples_ptr, InpBuff, k);
+    	//printf("mt63 rx: samples_remain: %d, samples_ptr: %p, i: %d\n", 
+	  //  samples_remain, samples_ptr, k);  			
+	samples_ptr += k/2;
+	samples_remain -= k/2;
+    } // else printf ("mt63rx: no demand of spectrum samples.\n");
+    
+    ConvS16toFloat(InpBuff,&InpFloat,InpLen);
+    /* printf("S16tofloat: %d samples\n", InpLen); */
+    /* is 512 always */
+    InpLevel.Process(&InpFloat);
+    if(UseRateConv){ 
+	RateConv.ProcessLinI(&InpFloat);
+	Rx.Process(&RateConv.Output);
+	printf("UseRateConv: Rx.Output.Len: %d chars\n", Rx.Output.Len);
+	// is 0 or 1
+    } else 
+	Rx.Process(&InpFloat);
+    //printf("Rx.Output.Len: %d chars\n", Rx.Output.Len); 
+    /* is 0 or 1 always, 
+        if 0, I should make the code buffer empty
+	to prevent double printed chars 
+     */
+    if (!Rx.Output.Len) memset(code, 0, sizeof(code));
+    for(i=0; i<Rx.Output.Len; i++) { 
+	code[i]=Rx.Output.Data[i];
+	/*
+	if((code[i]>=' ')||(code[i]==0x08)||(code[i]==0x09)) 
+          printf("%c2",code[i]);
+	
+	else if(code[i]=='\r') {
+          code[i] = '\n';
+	  printf("%c", code[i]);
+	}
+	else if(code[i]!='\0') {
+          printf("<%02X>",code[i]);
+	}
+	*/
+    }
+}
+
+// ============================================================================
+
 void mt63_finish_rx()  
 { 
   Rx.Free();
@@ -354,12 +434,16 @@ void mt63_direct_tx()
   char ch;
   char* testtext = "ich liebe dich von ganzem Herzen mit Schmerzen";  
 
-  printf("will initialize mt63 tx...\n");
+
   mt63_tx_start(1000, 0, 1.0, (unsigned char*)"MT63 ");
+  
   printf("will open audio for mt63 tx...\n");
   if (! open_audio_output("/dev/dsp"))
     printf("audio opened for mt63 test-tx.\n");
   
+  
+  
+  
 /* the following code is to debug the Tx.SendTune() */
 /*
   printf("Tuning ...\n");
@@ -384,12 +468,13 @@ void mt63_direct_tx()
       if(ch=='\r') printf("\n"); 
       fflush(stdout); 
     }
+    
     if(UseRateConv)
     { RateConv.ProcessLinI(&Tx.Comb.Output);
       ConvFloatToS16(&RateConv.Output,&OutBuff);
     } else ConvFloatToS16(&Tx.Comb.Output,&OutBuff);
     len=TxDev.Write(OutBuff.Data,OutBuff.Len);
-    printf("written to audio %d bytes", len);
+    //printf("written to audio %d bytes", len);
     if(len!=OutBuff.Len) { 
       printf("Error while sending out audio, errno => %s\n",strerror(errno));
       goto Stop; 
@@ -404,7 +489,7 @@ void mt63_direct_tx()
 	ConvFloatToS16(&RateConv.Output,&OutBuff);  } 
     else ConvFloatToS16(&Tx.Comb.Output,&OutBuff);
     len=TxDev.Write(OutBuff.Data,OutBuff.Len);
-    printf("written to audio %d bytes", len);
+    //printf("flushing: written to audio %d bytes", len);
     if(len!=OutBuff.Len) { 
 	printf("Error while sending out audio, errno => %s\n",strerror(errno));
 	goto Stop; 
@@ -419,7 +504,7 @@ void mt63_direct_tx()
       ConvFloatToS16(&RateConv.Output,&OutBuff);
     } else ConvFloatToS16(&Tx.Comb.Output,&OutBuff);
     len=TxDev.Write(OutBuff.Data,OutBuff.Len);
-    printf("written to audio %d bytes", len);
+    //printf("jam: written to audio %d bytes", len);
     if(len!=OutBuff.Len)  { 
 	printf("Error while sending out audio, errno => %s\n",strerror(errno));
 	goto Stop; 
@@ -427,7 +512,7 @@ void mt63_direct_tx()
   }
 
 Stop:
-  printf("\nClosing audio device ...\n");
+  printf("\nmt63hf.cc: Trying to close audio device DevNo ..\n");
   TxDev.Close();
   printf("Stopped OK.\n");
   printf("%ld samples sent out = %3.1f sec\n",
@@ -475,7 +560,7 @@ int mt63_encode(char ch) //, short* samples, int count)
 
 // ============================================================================
 
-void mt63_finish_transmit()  // not used
+void mt63_finish_tx() 
 { 
   int i, err, len, End; 
   printf("\nFlushing data interleaver ...\n");
@@ -486,7 +571,7 @@ void mt63_finish_transmit()  // not used
 	ConvFloatToS16(&RateConv.Output,&OutBuff);  } 
     else ConvFloatToS16(&Tx.Comb.Output,&OutBuff);
     len=TxDev.Write(OutBuff.Data,OutBuff.Len);
-    printf("written to audio %d bytes", len);
+    //printf("flushing: written to audio %d bytes", len);
     if(len!=OutBuff.Len) { 
 	printf("Error while sending out audio, errno => %s\n",strerror(errno));
 	exit (1); 
@@ -496,7 +581,67 @@ void mt63_finish_transmit()  // not used
 
 // ============================================================================
 
-void mt63_tx_send_jam() 
+void mt63_tx(char c) 
+{ 
+ // char* testtext = "ich liebe dich von ganzem Herzen mit Schmerzen";  
+ // char ch;
+  int i, k, len, End;
+/*
+      
+  printf("MT63 transmiting test text:\n");
+  for (i= 0; i < strlen(testtext); i++) {
+    ch= testtext[i]; Tx.SendChar(ch); if(ch==0x04) End=1;
+    if(ch) { 
+      printf("%c",ch);
+      if(ch=='\r') printf("\n"); 
+      fflush(stdout); 
+  }
+*/
+
+
+  //printf("MT63 transmiting character:\n");
+     
+    Tx.SendChar(c); 
+    if(c==0x04) End=1;
+    //if(c) { 
+      
+      if(UseRateConv) {
+	RateConv.ProcessLinI(&Tx.Comb.Output);
+        ConvFloatToS16(&RateConv.Output,&OutBuff);
+      } else 
+        ConvFloatToS16(&Tx.Comb.Output,&OutBuff);
+      
+      len=TxDev.Write(OutBuff.Data,OutBuff.Len);
+      //printf("written to audio %d bytes", len);
+         
+    // write samples to the buffer for the spectrum
+    if (samples_remain > 0) {
+    	k = 2*samples_remain;
+	if (k > BuffLen)
+	    k = BuffLen;
+	memcpy(samples_ptr, OutBuff.Data, k);
+    	//printf("mt63 tx: samples_remain: %d, samples_ptr: %p, i: %d\n", 
+	  //samples_remain, samples_ptr, k);  			
+	samples_ptr += k/2;
+	samples_remain -= k/2;
+    }  //else printf ("mt63tx: no demand of spectrum samples.\n");
+    
+
+     /*
+      printf("%c",c);
+      if(c=='\r') printf("\n"); 
+      fflush(stdout); 
+  */
+      if(len!=OutBuff.Len) { 
+        printf("Error while sending out audio, errno => %s\n",strerror(errno));
+//      goto Stop; 
+      }
+    //}
+}
+
+// ============================================================================
+
+void mt63_send_jam() 
 {
 	int i;
 	Tx.SendJam();
@@ -508,7 +653,7 @@ void mt63_tx_send_jam()
 	for(i = 0; i <= OutBuff.Len && i < ENCODEBUFSIZE; i++) {  
 	    encodebuf[i] = (short)OutBuff.Data[i];
 	}
-	// printf("written %d samples of jam to output buffer\n", i); 
+	printf("written %d samples of jam to output buffer\n", i); 
 	if(i < OutBuff.Len) { 
 	    printf("But only written %d of %d jam samples to output buffer\n", 
 		i, OutBuff.Len);
@@ -516,3 +661,28 @@ void mt63_tx_send_jam()
 		strerror(errno));
 	}
 }
+
+// ============================================================================
+
+void close_audio_rx() 
+{
+    printf("\nMT63: Closing audio device ...\n");
+    if (RxDev.Close()) printf("sound device could not be closed.\n");
+    printf("Stopped OK.\n");
+    printf("%ld samples sent out = %3.1f sec\n",
+     RxDev.TotalWritten,RxDev.TotalWritten/DevRate);
+}
+
+// ============================================================================
+
+void close_audio_tx() 
+{
+    printf("\nMT63: Closing audio device ...\n");
+    if (TxDev.Close()) printf("sound device could not be closed.\n");
+    printf("Stopped OK.\n");
+    printf("%ld samples sent out = %3.1f sec\n",
+     TxDev.TotalWritten,TxDev.TotalWritten/DevRate);
+}
+
+// ============================================================================
+
