@@ -33,17 +33,26 @@
 extern "C" {
 #endif /* __cplusplus */
 
+
+
 int mt63_rx_start(unsigned int bandwidth, unsigned int doubleinterleave, 
     unsigned int integration, float soundcorr);
 int mt63_tx_start(unsigned int bandwidth, unsigned int doubleinterleave, 
     float soundcorr, unsigned char* call);
+
+int open_audio_input(char *DevName);
+int open_audio_output(char *DevName);
 void mt63_decode(short *samples, int count);
 int mt63_encode(char ch); 
-void mt63_finish_tx();
-void mt63_finish_rx();
-void mt63_tx_send_jam(); 
+void mt63_tx(char c);
+void mt63_rx();
 void mt63_direct_rx(void);
 void mt63_direct_tx(void);
+void mt63_finish_tx();
+void mt63_finish_rx();
+void mt63_send_jam(); 
+void close_audio_rx();
+void close_audio_tx();
 
 #ifdef __cplusplus
 }
@@ -52,5 +61,11 @@ void mt63_direct_tx(void);
 extern char code[16];
 #define ENCODEBUFSIZE 1602 /* is maximal length, used for bandwidth 500 */
 extern short encodebuf[ENCODEBUFSIZE];
+extern int mt63_data_waiting;
+
+/* samples for the sprectrum */
+extern int samples_remain;
+extern int samples_count;
+extern short *samples_ptr;
 
 #endif /* MT63HF_H */
