@@ -90,6 +90,10 @@
 #include <unistd.h> // for usleep()
 #endif
 
+#ifdef __FreeBSD__ 
+#include <unistd.h>
+#endif
+
 // #ifdef __MSDOS__
 // #include <dos.h> // for delay()
 // #endif
@@ -357,7 +361,7 @@ printf("We will scan repetition times from %d to %d samples (after decim.)\n",
        MaxEnvLineDelay-(ScanRange-1),MaxEnvLineDelay);
 printf("and integrate the differences over %3.1f sec.\n",IntegTime);
 
-err=InpSplit.Preset(FilterLen,NULL,NULL,DecimRate);
+err=InpSplit.Preset(FilterLen,(float*)NULL,(float*)NULL,DecimRate);
 if(err) { printf("Can't preset the I/Q splitter - low memory ?\n"); goto Error; }
 err=InpSplit.ComputeShape(2*M_PI*(CenterFreq-BandWidth/2)/DevRate,
 			  2*M_PI*(CenterFreq+BandWidth/2)/DevRate,
