@@ -86,7 +86,6 @@ void on_frequency_spectrum_activate(GtkMenuItem *menuitem, gpointer user_data)
 	scope_on = 1;
 	gtk_widget_show(wspec);
 	return;
-	
 }
 
 gboolean on_spec_motion_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
@@ -775,17 +774,17 @@ gboolean on_textedit_button_press_event
 	case 3: /* dit */
 		msg.hdr.type = htonl(HFAPP_MSG_CW_ELBUG_DIT);
 		elbugstate = DIT;
-		display_status("cw dit");
+		//display_status("cw dit");
 		break;
-	case 2: /* tune */
+	case 2: /* tune */ /* UNUSED */
 		msg.hdr.type = htonl(HFAPP_MSG_CW_ELBUG_TUNE);
 		elbugstate = TUNE;
-		display_status("cw tune / tone ...");
+		//display_status("cw tune / tone ...");
 		break;
  	case 1: /* dah */
 		msg.hdr.type = htonl(HFAPP_MSG_CW_ELBUG_DAH);
 		elbugstate = DAH;
-		display_status("cw dah");
+		//display_status("cw dah");
 		break;
 	}
  	msg_send(&msg);
