@@ -31,6 +31,8 @@
 #include "mailbox.h"
 #include "hft.h"
 #include <sys/socket.h>
+#include <netinet/in.h>
+// if still error `SHUT_RDWR undeclared' try #include <arpa/inet.h>
 
 /* --------------------------------------------------------------------- */
 
@@ -137,6 +139,7 @@ void mailbox_input(unsigned char *data, int datalen) {
 	for (i = 0;  i < datalen && i < MAILBOX_IN_DATA_LEN; i++) {
 	    mailbox_in_data[i] = (char)data[i];
 	}
+
 /*
  *  to make the check easier, if our call is called, toupperize input 
  */	
@@ -161,9 +164,11 @@ void mailbox_input(unsigned char *data, int datalen) {
 		"%s", mailbox_in_data);
 	}
 	*/
+
 /*
  * write received mailbox_in_data to port-buffer:
  */	
+
 	sprintf(buf_hf2box + buf_hf2box_avail, "%s", mailbox_in_data);	
 	in_oldavail = buf_hf2box_avail; 
 	buf_hf2box_avail+= datalen;
@@ -176,10 +181,11 @@ void mailbox_input(unsigned char *data, int datalen) {
 */
 
 /*
- * special case: new pactor connect:
+ * special case: first round of new pactor connect:
  */	
 	if (lastrxmsg == HFAPP_MSG_STATE_PACTOR_ARQ_SLAVECONNECT &&
-	    mailbox_connected == 0) {
+	    mailbox_connected == 0
+	    && port == 6300 ) {
 /*
 	    display_status ( 
 	        "mailbox_input: Pactor ARQ slave connect! "
@@ -219,18 +225,20 @@ void mailbox_input(unsigned char *data, int datalen) {
  * case of FEC-signal, last sign is cr or nl:
  */	
 	if (buf_hf2box[buf_hf2box_avail -1 ] == '\r' ) {	
-//	    display_status ( 
-//	        "Last received char is carriage return -> "
-//		"will check mailbox.");       		
+/*
+	    display_status ( 
+	        "Last received char is carriage return -> "
+		"will check mailbox.");       		
+*/
 	    goto invoke_mailbox_check_io;
 	} 
 	else if (buf_hf2box[buf_hf2box_avail -1 ] == '\n' ) {	
-	    // this runs usually 
-	    /*
+	    // this happens usually 
+/*
 	    display_status ( 
 	        "Last received char is newline -> "
 		"will check mailbox.");  
-	    */
+*/
 	    goto invoke_mailbox_check_io;		     		
 	} 
 	else 
@@ -270,11 +278,11 @@ void mailbox_check_io() {
 	// the 'usual case' is:
 	// mailbox was already connected before, so read -> write
 	//    display_status 
-	//        ("mailbox_check_io: Will send input buf to mailbox");	
+	//        ("mailbox_check_io: Will send input buf to mailbox...");	
 	    if ( mailbox_test_port() ) {
 		mailbox_inbuf_2_port();
 	//    display_status 
-	//        ("mailbox_check_io: Will wait for answer from mailbox");	
+	//        ("mailbox_check_io: Waiting for answer from mailbox...");	
 		mailbox_port_2_outbuf();
 		return;
 	    } else {
@@ -290,55 +298,61 @@ void mailbox_check_io() {
 	    case HFAPP_MSG_STATE_PACTOR_ARQ_SLAVECONNECT:
 		display_status 
 		    ("mailbox_check_io: we have been connected via Pactor Slave"); 
-		display_status 
-		    ("by: %s", buf_hf2box); // + buf_hf2box_written); 
-		display_status 
-		    ("I will try to open the mailbox....");
-		if ( (i = mailbox_open_port()) < 0) {
+		if (port == 6300) {
+		    // very special case: Pactor 1 connect and
+		    // port == 6300 which means F6FBB.
+		    // Very special treatment.		
 		    display_status 
-		        ("mailbox_check_io: mailbox could not be opened.\n"
-		        "Please see if xfbbd is started and\n"
-		        "port and ipadress options are given correctly.");
-			mailbox_clear_allbufs();
+			("by: %s", buf_hf2box); // + buf_hf2box_written); 
+		    display_status 
+			("I will try to open the mailbox....");
+		    if ( (i = mailbox_open_port()) < 0) {
+			display_status 
+		    	    ("mailbox_check_io: mailbox could not be opened.\n"
+		    	    "Please see if xfbbd is started and\n"
+		    	    "port and ipadress options are given correctly.");
+			    mailbox_on = 0;		
+			    mailbox_clear_allbufs();
+			    display_status 
+				("I switched the mailbox mode off for now.");
 		        return;
-		}		    
-		mailbox_loggedin();
-		mailbox_clear_outbuf();
-		mailbox_port_2_outbuf();
-//		special case: discard the answer....
-		mailbox_clear_outbuf();
-//		display_status (  
+		    }		    
+		    mailbox_loggedin();
+		    mailbox_clear_outbuf();
+		    mailbox_port_2_outbuf();
+//			special case for f6fbb & pactor 1: discard the answer....
+		    mailbox_clear_outbuf();
+//			display_status (  
 //			"mailbox_check_io: "
 //			"buf_hf2box: %s", buf_hf2box);       
-//		display_status 
-//		    ("mailbox_check_io: pactor login."
-//		    "I discarded F6FBB's greeting and call question.");
+			display_status 
+		    ("mailbox_check_io: special case: pactor login, F6FBB.\n"
+		    "We got the remote user's call already.\n"
+		    "-> we discard F6FBB's greeting and call question.\n");
 		// user's call should stand in rx buffer now!!
-		mailbox_inbuf_2_port(); 
-		mailbox_port_2_outbuf();
+		    mailbox_inbuf_2_port(); 
+		    mailbox_port_2_outbuf();
     		// now the box got user's call and asks for password 
-		return;
+		    return;
+		} else {
+		// other case: port is not 6300, we do not use F6FBB.
+		    goto arq_connect;
+		    break;
+		}
 		break;
 
 	    case HFAPP_MSG_STATE_GTOR_ARQ_SLAVECONNECT:
+		display_status 
+		    ("mailbox_check_io: we have been connected via Gtor ARQ"); 
+		    goto arq_connect;
+		    break;
+		    
 	    case HFAPP_MSG_STATE_AMTOR_ARQ_SLAVECONNECT:
 		display_status 
-		    ("mailbox_check_io: we have been connected via Amtor or Gtor"); 
-		display_status 
-		    ("I will try to open the mailbox....");
-		if ( (i = mailbox_open_port()) < 0) {
-		    display_status 
-		        ("mailbox_check_io: mailbox could not be opened."
-		        "Please see if xfbbd is started and\n"
-		        "port and ipadress options are given correctly.");
-			mailbox_clear_allbufs();
-		        return;
-		}		    
-		mailbox_loggedin();
-		mailbox_clear_outbuf();
-		mailbox_port_2_outbuf();
-		return;
-		break;
+		    ("mailbox_check_io: we have been connected via Amtor ARQ"); 
+		    goto arq_connect;
+		    break;
+
 	    
 	    case HFAPP_MSG_STATE_GTOR_ARQ_DISCONNECT:
 	    case HFAPP_MSG_STATE_PACTOR_ARQ_DISCONNECT:
@@ -351,25 +365,25 @@ void mailbox_check_io() {
 	    case HFAPP_MSG_STATE_RTTY_RX:
 		display_status 
 		    ("mailbox_check_io: QRZ in RTTY ?"); 
-		goto goon;
+		goto fec_connect;
 		break;	
 
 	    case HFAPP_MSG_START_PACTOR_STANDBY: 
 		display_status 
 		    ("mailbox_check_io: QRZ in Pactor FEC?"); 
-		goto goon;
+		goto fec_connect;
 		break;
 	
 	    case HFAPP_MSG_START_STANDBY: 
 		display_status 
 		    ("mailbox_check_io: QRZ in Pactor/Amtor/Gtor FEC ?"); 
-		goto goon;
+		goto fec_connect;
 		break;
 	
 	    case HFAPP_MSG_STATE_PACTOR_FEC_CONNECT:
 		display_status 
 		    ("mailbox_check_io: QRZ in Pactor-FEC ?"); 
-		goto goon;
+		goto fec_connect;
 		break;
 	
 	    case HFAPP_MSG_STATE_AMTOR_FEC_CONNECT:
@@ -377,35 +391,64 @@ void mailbox_check_io() {
 	    case HFAPP_MSG_STATE_AMTOR_COLFEC_RX:
 		display_status 
 		    ("mailbox_check_io: QRZ in Amtor-FEC ?"); 
-		goto goon;
+		goto fec_connect;
 		break;	
 
 	    case HFAPP_MSG_STATE_MT63_RX:
 		display_status 
 		    ("mailbox_check_io: QRZ in MT63 ?"); 
-		goto goon;
+		goto fec_connect;
 		break;	
 
 	    default:
 		display_status("mailbox_check_io: QRZ ? "); 
-		goto goon;
+		goto fec_connect;
 	    break;
 	}
-	
-	goon:
-	/*look into the input buffer, check someone calling our call?*/
-	if (strstr(buf_hf2box, brag.call)) {
+
+arq_connect:
+	display_status 
+	    ("ARQ connect -> I will try to open the mailbox....");
+	if ( (i = mailbox_open_port()) < 0) {
+	/*
 	    display_status 
-		("mailbox_check_io: Was my call. Try to open mailbox....");
+	        ("mailbox_check_io: mailbox could not be opened.\n"
+	        "See if port and ipadress are given correctly.\n"
+		"Test what is waiting on the other side of the port,\n"
+		"e.g. by `telnet <host> <port> ` ... !");
+	*/
+	    mailbox_on = 0;		
+	    mailbox_clear_allbufs();
+	
+	    display_status 
+		("I switch the mailbox mode off for now.");
+	    mailbox_clear_allbufs();
+		return;
+	}		    
+	mailbox_loggedin();
+	mailbox_clear_outbuf();
+	mailbox_port_2_outbuf();
+	return;
+	
+fec_connect:
+	/*look into the input buffer, check someone calling our call?*/
+	if (strstr(buf_hf2box, params.brag.call)) {
+	    display_status 
+		("mailbox_check_io: Was my call. Try to open port...");
 	    if ( (i = mailbox_open_port()) < 0) {
 		display_status 
-		("mailbox_check_io: Could not open port to mailbox.");
+		("... could not open port to mailbox.\n"
+	        "See if port and ipadress are given correctly.\n"
+		"Test what program is waiting at the port,\n"
+		"e.g. by `telnet <host> <port> ` ... !");
+		mailbox_clear_allbufs();
 		return;
 	    }		    
 	    mailbox_loggedin();
-//	    display_status 
-//		("mailbox_check_io: New connect.....");
-	    mailbox_clear_outbuf();
+	    display_status 
+		("mailbox_check_io: New connect.....");
+	    mailbox_clear_allbufs();
+	    // first listen, like every ham should do... 
 	    mailbox_port_2_outbuf();
 	    return;
 	}
@@ -413,7 +456,6 @@ void mailbox_check_io() {
 	    display_status 
 	        ("mailbox_check_io: This was not my call.");
 	    mailbox_clear_allbufs();
-	    mailbox_input_waiting = 0;
 	    return;
 	}
 }
@@ -432,8 +474,6 @@ void mailbox_loggedin() {
 void mailbox_clear_allbufs() {
 	mailbox_clear_inbuf();
 	mailbox_clear_outbuf();
-	mailbox_input_waiting = 0;
-	mailbox_output_waiting = 0;
 //	display_status ( "cleared mailbox input & output buffers.");
 	return;
 }
@@ -443,6 +483,7 @@ void mailbox_clear_allbufs() {
 void mailbox_clear_inbuf() {
 	buf_hf2box_written = buf_hf2box_avail = in_oldavail = 0;
 	memset (buf_hf2box, 0, BUF_SIZE);
+	mailbox_input_waiting = 0;
 //	display_status ( "cleared mailbox input buffer.");
 	return;
 }
@@ -598,21 +639,21 @@ int mailbox_open_port() {
 	    return(0);
 	}
 
-	ipaddress = mailbox_host;
+	ipaddress = params.mailbox.host;
 	if (ipaddress == NULL) {
 	    display_status (  
-		"mailbox_open: You did not specify the option '-h' \n"
-		"(IP address of the mailbox computer) for hfterm.\n"
-		"I will use '127.0.0.1' (localhost) as default.");
+		"mailbox_open: You did not specify option '-h' \n"
+		"(IP address of mailbox computer) for hfterm.\n"
+		"Will use '127.0.0.1' (localhost) as default.");
 	    ipaddress = "127.0.0.1";
 	}
 
-	port = mailbox_port;
+	port = params.mailbox.port;
 	if (port == 0) {
 	    display_status (  
-		"mailbox_open: You did not specify the option '-p'\n"
+		"mailbox_open: You did not specify option '-p'\n"
 		"(port at which the mailbox can be connected),\n"
-		"I will use 6300, which is the default in F6FBB.");
+		"Will use 6300, which is default for F6FBB.");
 	    port = 6300;
 	}
 
@@ -630,9 +671,9 @@ int mailbox_open_port() {
 		( "I cleared buffers and switched mailbox mode off.");
 	    return(-1);
 	} else {
-//	    display_status 
-//		( "mailbox_open: ready port to f6fbb mailbox: %s:%d", 
-//		ipaddress, port);	
+	    display_status 
+		( "mailbox_open: ready port %s:%d", 
+		ipaddress, port);	
 	    return(0);
 	}
 }
@@ -752,7 +793,7 @@ void mailbox_inbuf_2_port () {
 
 	if (mailbox_fd < 0) {
 	    display_status (  
-		"mailbox_inbuf_2_port: Sorry, port %s:%d not reached.", 
+		"mailbox_inbuf_2_port: Sorry, not reached port %s:%d.", 
 		ipaddress, port);
 	    return;
 	}
@@ -847,7 +888,7 @@ void mailbox_port_2_outbuf () {
 
 	if (mailbox_fd < 0) {
 	    display_status (  
-            	"mailbox_port_2_outbuf: Sorry, port %s:%d could not be opened.", 
+            	"mailbox_port_2_outbuf: Sorry, port %s:%d unreachable.", 
 		ipaddress, port);
 	    mailbox_loggedout();
 	    return;
@@ -895,7 +936,7 @@ void mailbox_port_2_outbuf () {
 		BUF_SIZE - buf_box2hf_avail);
 	    if (newtxbytes == 0) {
 		display_status 
-		    ( "mailbox_port_2_outbuf: now nothing to read from mailbox");       
+		    ( "mailbox_port_2_outbuf: nothing to read from port.");       
 		mailbox_input_waiting = 0;
 		return;
     	    } else if (newtxbytes < 0) {
@@ -956,7 +997,7 @@ void mailbox_loggedout() {
 	mailbox_connected = 0;
 	beacon_restore();
 	mailbox_on = 1;
-//	means mailbox_on stays 1! Standby shall remain for automatic mailbox!!!
+//	mailbox_on stays 1, means still standby of automatic mailbox!!!
 //	can be set to 0 only by menu via function on_mailbox_activate 
 //	in callbacks.c.
 //	transmit("Mailbox closed -> disconnect! 73 de hfterm!\n");
