@@ -39,7 +39,9 @@
 /* --------------------------------------------------------------------- */
 
 char versioninfo[128];
+char kernelstartcommand[128];
 int rxfile_ready = 0, scope_on = 0, log_on = 0;
+int kernelstart = 1;
 static int fd_krnl = -1;
 static char *name_kernel = "/var/run/hfapp";
 char gmt[32];
@@ -132,9 +134,15 @@ void write_input(unsigned char *data, int datalen)
 	
 	if (!data && datalen <= 0) return;
 
-	/* squelch. The squelch_passed... flags are updated in spectrum.c */
+	/* squelch. The squelch_passed... flags are updated in spectrum.c 
+	 * squelch works only if the spectrum is on! 
+	 */
+	
+	// no squelch for mt63, as no spectrum working
 	if (lastrxmsg == HFAPP_MSG_STATE_MT63_RX) {
-	    if (!squelch_passed_mt36_500_1000) {
+	  if (scope_on) {
+/*
+	      if (!squelch_passed_mt36_500_1000) {
 		if (params.mt63.doubleinterleave) factor = 2;
 		else factor = 1;
 		squelchwait = (params.mt63.integration * factor) ; 
@@ -142,19 +150,22 @@ void write_input(unsigned char *data, int datalen)
 		//display_status("MT63 squelch waits for %d units", squelchwait);
 		//display_status("MT63 squelch count %d units", squelchcount);
 		if (squelchcount > squelchwait) {
-		    /*
-		    display_status(
-			"MT63 signal remained weak for %d units -> squelched",
-			squelchwait);
-		    */
-		    return;
+		//	display_status(
+		//	"MT63 signal remained weak for %d units -> squelched",
+		//	squelchwait);
+	       return;
 		}
-	    } else squelchcount = 0;
+
+	      } else squelchcount = 0;
+*/	      
+	  }
 	}
+
 	/* for cw mode under construction */
-	/*
 	else if ((lastrxmsg == HFAPP_MSG_STATE_CW_RX) &&
 	    ( ! squelch_passed_cw)) {
+	  if (scope_on) {
+/*	  
 	    squelchwait = 10;
 	    squelchcount++;
 	    if (squelchcount > squelchwait ) {
@@ -162,9 +173,13 @@ void write_input(unsigned char *data, int datalen)
 		params.mt63.integration * factor);
 		return;
 	    }
+*/	    
+	  }
 	}
-	*/
-	else if (!squelch_passed_fsk) {
+
+	/* for all other modes, which are the fsk modes */
+	else if (scope_on) {
+	    if (!squelch_passed_fsk) {
 		squelchwait = 0;
 		squelchcount++;
 		//display_status("FSK squelch waits for %d units", squelchwait);
@@ -175,9 +190,11 @@ void write_input(unsigned char *data, int datalen)
 			"FSK signal remained weak for %d units -> squelched",
 			squelchwait);
 		    */
+		    display_status("FSK signal weak since %d units -> squelched",	squelchwait);
 		    return;
 		}
-	} else squelchcount = 0;
+	    } else squelchcount = 0;
+	}
 	
 	fbbtest = 0;
 	if (mailbox_on == 1) mailbox_input(data, datalen);
@@ -519,9 +536,9 @@ void timequery()
 	time(&now);
 	// "GENORMT":
 	//sprintf(gmt, "%s", asctime(gmtime(&now)));	
-	// MIR GEFLLT ES SO BESSER:
+	// This i like better:
 	// strftime(gmt, 31, "%a %d.%m.%y, %H:%M", gmtime(&now));
-	// aber damit es cabrillo-kompatibel wird mach ich es so:
+	// but to makie it compatible to cabrillo:
 	strftime(gmt, 31, "%Y-%m-%d %H%M", gmtime(&now));
 } 
 
@@ -530,48 +547,92 @@ void init(void)
 {
 	struct sockaddr_un saddr;
 	int kernel_running = 0;
-	struct hfapp_msg msg;
-
-	system("cd");
-	system("if ! [ -d hf ] ; then mkdir hf; cp -ruv /usr/share/hf/hf-examplefiles/* hf; fi");
-	sprintf(versioninfo,  "hfterm %s by Tom Sailer, Axel Krause, "
-	    "Gnther Montag, Pawel Jalocha et al. ", VERSION);
+	char systemstring[256];
+	
+	system("cd ~"); 
+	system("pwd"); 
+		sprintf(systemstring, 
+	 	"if ! [ -d ~/hf ] ; then mkdir ~/hf; cp -ruv %s/share/hf/hf-examplefiles/* ~/hf; fi", PREFIX);
+	system(systemstring);
+	
+	sprintf(versioninfo,  "hf %s. All Docs in: %s/share/hf", 
+	    PACKAGE_VERSION, PREFIX);
 	display_status(versioninfo);
-	if ((fd_krnl = socket(PF_UNIX, SOCK_STREAM, 0)) < 0)
-		errstr(SEV_FATAL, "socket");
-	saddr.sun_family = AF_UNIX;
-	strncpy(saddr.sun_path, name_kernel, sizeof(saddr.sun_path));
-	if (connect(fd_krnl, (struct sockaddr *)&saddr, sizeof(saddr))) {
-		errstr(SEV_WARNING, "connect");
-		close(fd_krnl);
-		msgpfd.fd = fd_krnl = -1;
-		display_status("This is hfterm. hfkernel is not started!\n"
-		    "To start both, use the start script hf.\n"
-		    "Maybe you have to adapt /etc/hf.conf first. \n"
-		    "(e.g. correct the ttyS.. (serial port for PTT)  entry.)\n"
-		    "See F1 or /usr/share<doc/>(packages/)/hf/HF-HOWTO for help.");
-	} else {
-		kernel_running = 1;
-		g_source_add(G_PRIORITY_HIGH, FALSE, 
-		    &poll_funcs, NULL, NULL, NULL);
-		msgpfd.fd = fd_krnl;
-		msgpfd.events = G_IO_IN;
-		msgpfd.revents = 0;
-		g_main_add_poll(&msgpfd, G_PRIORITY_HIGH);
-	}
-
+	
 	param_set_defaults();
 	param_read();	
 	fixtext_read();
 	log_read();
 	log_list();
 	param_set();
-	param_kernel();
 	logbook_window_show(NULL, NULL	);
 	if ((radiofont = gdk_font_load("7x13")) == NULL)
 	    display_status("simple font 7x13 could not be loaded.");
+		
+	if(kernelstart) {
+	    sprintf(kernelstartcommand, 
+		"hfkernel -s %f -t %f -m %f -a %s -p %s %s &",
+		 params.general.snd_corr, 
+		 params.general.tv_corr, 
+		 params.general.cpu_mhz, 
+		 params.general.soundcard, 
+		 params.general.serial, 
+		 params.general.kerneloptions);
+	
+		printf(" the hfkernel start command is: \n %s\n", 		kernelstartcommand);
+	
+	// I kill all hfkernel threads first to go sure no zombies around there	
+	    system("hfkernel -k");
 	    
+	// I start hfkernel new
+	    display_status("I start hfkernel now....");
+	    system(kernelstartcommand);
+	    display_status("I will wait a second....");
+	    sleep(1);
+	    display_status("I will try to connect...");
+	}
+	
+	// I connect the kernel
+	if ((fd_krnl = socket(PF_UNIX, SOCK_STREAM, 0)) < 0)
+		errstr(SEV_FATAL, "socket");
+	saddr.sun_family = AF_UNIX;
+	strncpy(saddr.sun_path, name_kernel, sizeof(saddr.sun_path));
+	if (connect(fd_krnl, (struct sockaddr *)&saddr, sizeof(saddr)))
+	{
+	    errstr(SEV_WARNING, "connect");
+	    close(fd_krnl);
+	    msgpfd.fd = fd_krnl = -1;
+	    if (kernelstart) {
+		display_status
+		    ("I could not start hfkernel.\n"
+		    "Please try to start it on a console first.\n" 
+		    "Write down the options you need, \n"
+		    "enter them into the configure menu.\n"
+		    "See F1 or /usr/share<doc/>(packages/)/hf/HF-HOWTO for help.\n"); 
+	    } else {
+	        display_status
+		    ("By option -n you did not want to start hfkernel.\n"
+		    "It also seems not to have been running before.\n" 
+		    "This mode is for debug only.\n" );
+	    }
+	} else {
+		// if the connect is o.k.
+		
+		g_source_add(G_PRIORITY_HIGH, FALSE, 
+		    &poll_funcs, NULL, NULL, NULL);
+		msgpfd.fd = fd_krnl;
+		msgpfd.events = G_IO_IN;
+		msgpfd.revents = 0;
+		g_main_add_poll(&msgpfd, G_PRIORITY_HIGH);
+		kernel_running = 1;
+		display_status
+		    ("hfterm has connected hfkernel All is good.\n" );
+	}
+	    	
 	if(kernel_running) {
+	    param_kernel();
+	
+	/*
 	    msg.hdr.err = htonl(ERR_NOERR);
 	    msg.hdr.len = htonl(0);
 	    msg.hdr.type = htonl(HFAPP_MSG_START_STANDBY);
@@ -579,6 +640,7 @@ void init(void)
 	    display_status("start with STANDBY FOR PACTOR-AMTOR- GTOR...");
 	    lastrxcmd = HFAPP_MSG_START_STANDBY;
 	    way = RX;
+	*/
 	}
 
 }
@@ -599,6 +661,8 @@ void finit(void)
 	param_store();
 /* - Log-Datei speichern ---------------------------------------------*/
 	log_store();
+/* - finish hfkernel     ---------------------------------------------*/
+	system ("hfkernel -k");
 }
 
 
@@ -613,10 +677,14 @@ int main(int argc, char *argv[])
 //		
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
-
-        while ((c = getopt(argc, argv, "c:h:p:")) != -1) 
+	
+        while ((c = getopt(argc, argv, "nc:h:p:")) != -1) 
                 switch (c) {
-                case 'c':
+                case 'n':
+                	kernelstart = 0;
+			break;
+
+		case 'c':
                 	if (optarg != NULL) name_kernel = optarg;
                         break;
 
@@ -635,9 +703,9 @@ int main(int argc, char *argv[])
         if (err) {
                 errprintf 
 		    (SEV_WARNING,
-		    "usage: hfterm [-c <socket>] [-p <port>] [-h <host>]\n"
-		    "e.g.:\n"
-		    " -c: socket to hfkernel, default: /var/run/hfapp"
+		    "usage: hfterm [n] [-c <socket>] [-p <port>] [-h <host>]\n"
+		    " -n: no start of hfkernel (debug, graphical interface only)\n"
+		    " -c: socket to hfkernel, default: /var/run/hfapp\n"
 		    " -h: ip-adress of mailbox-host, default: 127.0.0.1\n"
 		    " -p  port of mailbox programm, default: 6300\n");
                 exit(1);
@@ -671,7 +739,7 @@ int main(int argc, char *argv[])
 	wmonitor = create_wmonitor();
 	gtk_widget_show(wmain);
 
- 	init();
+	init();
 	gtk_main();
 	exit(0);
 }
