@@ -156,9 +156,7 @@ create_wmain (void)
   gtk_widget_set_name (wmain, "wmain");
   gtk_object_set_data (GTK_OBJECT (wmain), "wmain", wmain);
   
-  sprintf(versiontitle, 
-      "hf %s - ham radio program with CW-elbug, FSK ARQ modes, MT63, TCP-mailbox interface", 
-      VERSION);
+  sprintf(versiontitle, "hf %s", PACKAGE_VERSION);
 
   gtk_window_set_title (GTK_WINDOW (wmain), _(versiontitle));
   gtk_window_set_position (GTK_WINDOW (wmain), GTK_WIN_POS_CENTER);
