@@ -67,19 +67,26 @@ void param_set_defaults(void)
 /* Falls keine config-Datei gefunden wird: params vorbelegen------- */
 /* if binary per-user config ~/hfrc not fount, preset params ------ */
 	char mycall[16];
-	char buf[16];
+	char buf[1024];
 	char *bp1;
 	const char *bp2;
 	
 	strncpy(mycall, MYCALL, sizeof(mycall));
+	
 	memset(&params, 0, sizeof(params));
+	
 	strncpy(params.brag.call, MYCALL, sizeof(params.brag.call));
+	params.general.snd_corr = 1.0;
+	params.general.tv_corr = 1.0;
+	params.general.cpu_mhz = 0;
+	strncpy(params.general.soundcard, "/dev/dsp0", sizeof(params.general.soundcard));
+	strncpy(params.general.serial, "none", sizeof(params.general.serial));
 	params.general.beaconpause = 10;
 	params.general.squelchpercent = 5;
 	params.cw.wpm = 12;
 	params.cw.tone = 880;
 	params.cw.farnsworth = 0;
-	params.rtty.baud = 45;
+	params.rtty.baud = 45.45;
 	params.amtor.txdelay = 30;
 	params.amtor.retry = 30;
 	params.gtor.txdelay = 30;
@@ -123,7 +130,8 @@ void param_get(void)
 {
 	GtkEntry *entry;
 	GtkToggleButton *tog;
-
+	char par[64];
+	
 	/* brag */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "entry27"));
 	strncpy(params.brag.call, gtk_entry_get_text(entry), sizeof(params.brag.call));
@@ -150,7 +158,34 @@ void param_get(void)
 
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "entry34"));
 	strncpy(params.brag.www, gtk_entry_get_text(entry), sizeof(params.brag.www));
- 
+ 	
+	/* general */
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "soundcorr"));
+	params.general.snd_corr = strtod(gtk_entry_get_text(entry), NULL);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "timecorr"));
+	params.general.tv_corr = strtod(gtk_entry_get_text(entry), NULL);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "cpumhzcorr"));
+	params.general.cpu_mhz = strtod(gtk_entry_get_text(entry), NULL);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar),
+	"soundcardvalue"));
+	strncpy(par, gtk_entry_get_text(entry), sizeof(par));
+	if (!par[0]) sprintf(par, "none");
+ 	strncpy(params.general.soundcard, par, sizeof(par));
+ 	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar),
+	"serialvalue"));
+	strncpy(par, gtk_entry_get_text(entry), sizeof(par));
+	if (!par[0]) sprintf(par, "none");
+ 	strncpy(params.general.serial, par, sizeof(par));
+ 	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar),
+	"kerneloptions"));
+	strncpy(params.general.kerneloptions, gtk_entry_get_text(entry), sizeof(params.general.kerneloptions));
+ 	
+	
 	/* FSK parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "fskspacefreq"));
 	params.fsk.freq[0] = strtoul(gtk_entry_get_text(entry), NULL, 0);
@@ -158,6 +193,7 @@ void param_get(void)
 	params.fsk.freq[1] = strtoul(gtk_entry_get_text(entry), NULL, 0);
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "squelchpercent"));
 	params.general.squelchpercent = strtoul(gtk_entry_get_text(entry), NULL, 0);
+	
 	/* CW parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "wpm"));
 	params.cw.wpm = strtoul(gtk_entry_get_text(entry), NULL, 0);
@@ -167,15 +203,17 @@ void param_get(void)
 	params.cw.farnsworth = gtk_toggle_button_get_active(tog);
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "dtr"));
 	params.cw.dtr = gtk_toggle_button_get_active(tog);
+	
 	/* RTTY parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "rttybaudrate"));
-	params.rtty.baud = strtoul(gtk_entry_get_text(entry), NULL, 0);
+	params.rtty.baud = strtod(gtk_entry_get_text(entry), NULL);
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "rttyinvert"));
 	params.rtty.rxinvert = gtk_toggle_button_get_active(tog);
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "rttyrxtxinvert"));
 	params.rtty.txinvert = gtk_toggle_button_get_active(tog);
 	if (params.rtty.rxinvert)
 		params.rtty.txinvert = !params.rtty.txinvert;
+	
 	/* Amtor parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "amtordestcall"));
 	strncpy(params.amtor.destcall, gtk_entry_get_text(entry), sizeof(params.amtor.destcall));
@@ -194,6 +232,7 @@ void param_get(void)
 	params.amtor.txinvert = gtk_toggle_button_get_active(tog);
 	if (params.amtor.rxinvert)
 		params.amtor.txinvert = !params.amtor.txinvert;
+	
 	/* GTOR parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "gtordestcall"));
 	strncpy(params.gtor.destcall, gtk_entry_get_text(entry), sizeof(params.gtor.destcall));
@@ -204,6 +243,7 @@ void param_get(void)
 	if (params.gtor.retry < 4) params.gtor.retry = 4;
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "gtortxdelay"));
 	params.gtor.txdelay = strtoul(gtk_entry_get_text(entry), NULL, 0);
+	
 	/* Pactor parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorcall"));
 	strncpy(params.pactor.destcall, gtk_entry_get_text(entry), sizeof(params.pactor.destcall));
@@ -224,6 +264,7 @@ void param_get(void)
 	params.pactor.crcpreset[2] = strtoul(gtk_entry_get_text(entry), NULL, 16);
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorcrc3"));
 	params.pactor.crcpreset[3] = strtoul(gtk_entry_get_text(entry), NULL, 16);
+	
 	/* MT63 parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "mt63_bandwidth_value"));
 	params.mt63.bandwidth = strtoul(gtk_entry_get_text(entry), NULL, 0);
@@ -233,6 +274,7 @@ void param_get(void)
 	strncpy(params.mt63.cwcall, gtk_entry_get_text(entry), sizeof(params.mt63.cwcall));
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "mt63_doubleinterleave"));
 	params.mt63.doubleinterleave = gtk_toggle_button_get_active(tog);
+	
 	/* mailbox parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "mailbox_host"));
 	strncpy(params.mailbox.host, gtk_entry_get_text(entry), sizeof(params.mailbox.host));
@@ -245,7 +287,7 @@ void param_set(void)
 {
 	GtkEntry *entry;
 	GtkToggleButton *tog;
-	char buf[129];
+	char buf[256];
 
 	/* brag */
 
@@ -289,120 +331,197 @@ void param_set(void)
 	buf[sizeof(params.brag.www)] = 0;
 	gtk_entry_set_text(entry, buf);
  
+	/* general */
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "soundcorr"));
+	snprintf(buf, sizeof(buf), "%f", params.general.snd_corr);
+	gtk_entry_set_text(entry, buf);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "timecorr"));
+	snprintf(buf, sizeof(buf), "%f", params.general.tv_corr);
+	gtk_entry_set_text(entry, buf);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "cpumhzcorr"));
+	snprintf(buf, sizeof(buf), "%f", params.general.cpu_mhz);
+	gtk_entry_set_text(entry, buf);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar),
+	"soundcardvalue"));
+	strncpy(buf, params.general.soundcard, sizeof(params.general.soundcard));
+	buf[sizeof(params.general.soundcard)] = 0;
+	gtk_entry_set_text(entry, buf);
+	 	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar),
+	"serialvalue"));
+	strncpy(buf, params.general.serial, sizeof(params.general.serial));
+	buf[sizeof(params.general.serial)] = 0;
+	gtk_entry_set_text(entry, buf);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar),
+	"kerneloptions"));
+	strncpy(buf, params.general.kerneloptions, sizeof(params.general.kerneloptions));
+	buf[sizeof(params.general.kerneloptions)] = 0;
+	gtk_entry_set_text(entry, buf);
+		
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "beaconpausevalue"));
+	snprintf(buf, sizeof(buf), "%u", params.general.beaconpause);
+	gtk_entry_set_text(entry, buf);
+	
+	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "squelchpercent"));
+	snprintf(buf, sizeof(buf), "%u", params.general.squelchpercent);
+	gtk_entry_set_text(entry, buf);
+		
 	/* FSK parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "fskspacefreq"));
 	snprintf(buf, sizeof(buf), "%u", params.fsk.freq[0]);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "fskmarkfreq"));
 	snprintf(buf, sizeof(buf), "%u", params.fsk.freq[1]);
 	gtk_entry_set_text(entry, buf);
-	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "squelchpercent"));
-	snprintf(buf, sizeof(buf), "%u", params.general.squelchpercent);
-	gtk_entry_set_text(entry, buf);
+	
 	/* CW parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "wpm"));
 	snprintf(buf, sizeof(buf), "%u", params.cw.wpm);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "tone"));
 	snprintf(buf, sizeof(buf), "%u", params.cw.tone);
 	gtk_entry_set_text(entry, buf);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "farnsworth"));
 	gtk_toggle_button_set_active(tog, params.cw.farnsworth);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "dtr"));
 	gtk_toggle_button_set_active(tog, params.cw.dtr);
+	
+	
 	/* RTTY parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "rttybaudrate"));
-	snprintf(buf, sizeof(buf), "%u", params.rtty.baud);
+	snprintf(buf, sizeof(buf), "%f", params.rtty.baud);
 	gtk_entry_set_text(entry, buf);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "rttyinvert"));
 	gtk_toggle_button_set_active(tog, params.rtty.rxinvert);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "rttyrxtxinvert"));
 	gtk_toggle_button_set_active(tog, params.rtty.txinvert ^ params.rtty.rxinvert);
+	
+	
 	/* Amtor parameters */
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "amtordestcall"));
 	strncpy(buf, params.amtor.destcall, sizeof(params.amtor.destcall));
 	buf[sizeof(params.amtor.destcall)] = 0;
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "amtormycall"));
 	strncpy(buf, params.amtor.mycall, sizeof(params.amtor.mycall));
 	buf[sizeof(params.amtor.mycall)] = 0;
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "amtorselfeccall"));
 	strncpy(buf, params.amtor.selfeccall, sizeof(params.amtor.selfeccall));
 	buf[sizeof(params.amtor.selfeccall)] = 0;
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "amtortxdelay"));
 	snprintf(buf, sizeof(buf), "%u", params.amtor.txdelay);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "amtorretry"));
 	snprintf(buf, sizeof(buf), "%u", params.amtor.txdelay);
 	gtk_entry_set_text(entry, buf);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "amtorinvert"));
 	gtk_toggle_button_set_active(tog, params.amtor.rxinvert);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "amtorrxtxinvert"));
 	gtk_toggle_button_set_active(tog, params.amtor.txinvert ^ params.amtor.rxinvert);
+	
+	
 	/* GTOR parameters */
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "gtordestcall"));
 	strncpy(buf, params.gtor.destcall, sizeof(params.gtor.destcall));
 	buf[sizeof(params.gtor.destcall)] = 0;
 	gtk_entry_set_text(entry, buf);
+	
 	//entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "gtormycall"));
 	//strncpy(buf, params.gtor.mycall, sizeof(params.gtor.mycall));
 	//buf[sizeof(params.gtor.mycall)] = 0;
 	//gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "gtorretry"));
 	snprintf(buf, sizeof(buf), "%u", params.gtor.retry);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "gtortxdelay"));
 	snprintf(buf, sizeof(buf), "%u", params.gtor.txdelay);
 	gtk_entry_set_text(entry, buf);
 	/* Pactor parameters */
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorcall"));
 	strncpy(buf, params.pactor.destcall, sizeof(params.pactor.destcall));
 	buf[sizeof(params.pactor.destcall)] = 0;
 	gtk_entry_set_text(entry, buf);
+	
 	//entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactormycall"));
 	//strncpy(buf, params.pactor.mycall, sizeof(params.pactor.mycall));
 	//buf[sizeof(params.pactor.mycall)] = 0;
 	//gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorretry"));
 	snprintf(buf, sizeof(buf), "%u", params.pactor.retry);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactortxdelay"));
 	snprintf(buf, sizeof(buf), "%u", params.pactor.txdelay);
 	gtk_entry_set_text(entry, buf);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "pactorlongpath"));
 	gtk_toggle_button_set_active(tog, params.pactor.longpath);
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorcrc0"));
 	snprintf(buf, sizeof(buf), "%04X", params.pactor.crcpreset[0]);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorcrc1"));
 	snprintf(buf, sizeof(buf), "%04X", params.pactor.crcpreset[1]);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorcrc2"));
 	snprintf(buf, sizeof(buf), "%04X", params.pactor.crcpreset[2]);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "pactorcrc3"));
 	snprintf(buf, sizeof(buf), "%04X", params.pactor.crcpreset[3]);
 	gtk_entry_set_text(entry, buf);
+	
+	
 	/* MT63 parameters */
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "mt63_bandwidth_value"));
 	snprintf(buf, sizeof(buf), "%u", params.mt63.bandwidth);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "mt63_integration"));
 	snprintf(buf, sizeof(buf), "%u", params.mt63.integration);
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "mt63_cwcall"));
 	strncpy(buf, params.mt63.cwcall, sizeof(params.mt63.cwcall));
 	buf[sizeof(params.mt63.cwcall)] = 0;
 	gtk_entry_set_text(entry, buf);
+	
 	tog = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(wpar), "mt63_doubleinterleave"));
 	gtk_toggle_button_set_active(tog, params.mt63.doubleinterleave);
+	
+	
 	/* mailbox parameters */
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "mailbox_host"));
 	strncpy(buf, params.mailbox.host, sizeof(params.mailbox.host));
 	buf[sizeof(params.mailbox.host)] = 0;
 	gtk_entry_set_text(entry, buf);
+	
 	entry = GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(wpar), "mailbox_port"));
 	snprintf(buf, sizeof(buf), "%d", params.mailbox.port);
 	gtk_entry_set_text(entry, buf);
@@ -411,6 +530,7 @@ void param_set(void)
 //static 
 void param_kernel(void)
 {
+	/* we send the params to hfkernel */
 	struct hfapp_msg msg;
 	/* general parameters */
 	msg.hdr.type = htonl(HFAPP_MSG_SET_GENERALPAR);
@@ -433,7 +553,8 @@ void param_kernel(void)
 	msg.hdr.type = htonl(HFAPP_MSG_SET_RTTYPAR);
 	msg.hdr.err = htonl(ERR_NOERR);
 	msg.hdr.len = htonl(sizeof(msg.data.rpar));
-	msg.data.rpar.baud = htons(params.rtty.baud);
+	//msg.data.rpar.baud = htons(params.rtty.baud);
+	msg.data.rpar.baud = (params.rtty.baud);
 	msg.data.rpar.rxinvert = params.rtty.rxinvert;
 	msg.data.rpar.txinvert = params.rtty.txinvert;
 	msg_send(&msg);
@@ -489,15 +610,15 @@ void param_read()
 	FILE *conf;
 	conf = fopen("hf/hfterm.rc", "r");
 	if(conf == NULL) {
-	  display_status("Configuration File ~/hf/hfterm.rc can not be opened."
-	    "Will keep defaults.");
+	  display_status("Configuration File hf/hfterm.rc can not be opened.");
 	  return;
 	}  
 	else	
-//	display_status ("Configuration File ~/hf/hfterm.rc opened.");
+	
+	display_status ("Configuration File hf/hfterm.rc opened.");
 	if(fread(&params, sizeof(params), 1, conf) != 1) {
-		display_status("Error while reading configuration file."
-		    "Will keep defaults.");
+		display_status("Error while reading configuration file.");
+		fclose(conf);	
 		return;
 	}
 	fclose(conf);	
@@ -507,22 +628,20 @@ void param_store()
 {
 /* - Konfiguration speichern ----------------------------------------*/
 	FILE *conf = fopen("hf/hfterm.rc", "w");
-	if(conf == NULL)
-	{
+	if(conf == NULL) {
 	  display_status
-	      ("Configuration File ~/hf/hfterm.rc can not be opened.");
+	      ("Configuration File hf/hfterm.rc can not be opened.");
 	  return;
-	}
-	else	
-	    //display_status("Configuration File ~/hf/hfterm.rc opened.");
+	} else	
+	    display_status("Configuration File hf/hfterm.rc opened.");
 	if(fwrite(&params, sizeof(params), 1, conf) != 1)
 	{
 	  display_status
-	      ("Error while writing configuration file ~/hfterm.rc.");
+	      ("Error while writing configuration file hf/hfterm.rc.");
 	  return;
 	}
 	else	
-//	display_status("Configuration stored in ~/hf/hfterm.rc.");
+	display_status("Configuration stored in hf/hfterm.rc.");
 	fclose(conf);	
 }
 
