@@ -41,13 +41,37 @@ create_wpar (void)
 */
   GtkWidget *label_general;
   GtkWidget *table_general;
-  GtkWidget *label_squelch;
-  GtkWidget *squelchpercent;
+  
+  GtkWidget *label_soundcorr;
+  GtkWidget *soundcorr;
+  
+  GtkWidget *label_timecorr;
+  GtkWidget *timecorr;
+  
+  GtkWidget *label_cpumhzcorr;
+  GtkWidget *cpumhzcorr;
+  
+  GtkWidget *label_soundcard;
+  GtkWidget *soundcard;
+  GList *soundcard_items = NULL;
+  GtkWidget *soundcardvalue;
+  
+  GtkWidget *label_serial;
+  GtkWidget *serial;
+  GList *serial_items = NULL;
+  GtkWidget *serialvalue;
+  
+  GtkWidget *label_kerneloptions;
+  GtkWidget *kerneloptions;
+  
   GtkWidget *label_beaconpause;
   GtkWidget *beaconpause;
   GList *beaconpause_items = NULL;
   GtkWidget *beaconpausevalue;
-
+  
+  GtkWidget *label_squelch;
+  GtkWidget *squelchpercent;
+    
   GtkWidget *table_fsk;
   GtkWidget *label_mark;
   GtkWidget *label_space;
@@ -150,7 +174,10 @@ create_wpar (void)
   GtkWidget *parok;
   GtkWidget *parcancel;
   GtkAccelGroup *par_accels;
-
+  
+  GtkTooltips *tooltips;
+  tooltips = gtk_tooltips_new ();  
+  
   wpar = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_name (wpar, "wpar");
   gtk_object_set_data (GTK_OBJECT (wpar), "wpar", wpar);
@@ -395,7 +422,7 @@ create_wpar (void)
                     (GtkAttachOptions) (0), 0, 0);
 
 /* General Settings */
-  table_general = gtk_table_new (2, 2, FALSE);
+  table_general = gtk_table_new (17, 2, FALSE);
   gtk_widget_set_name (table_general, "table_general");
   gtk_widget_ref (table_general);
   gtk_object_set_data_full (GTK_OBJECT (wpar), "table_general", table_general,
@@ -411,13 +438,194 @@ create_wpar (void)
   gtk_widget_show (label_general);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (paramnotebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (paramnotebook), 1), label_general);
 
+  
+  gtk_tooltips_set_tip (tooltips, label_general, _("Note that you have to restart hfterm to make these basic settings work (except beacon pause and squelch percent)."), NULL);
+  gtk_tooltips_set_tip (tooltips, table_general, _("Note that you have to restart hfterm to make these basic settings work (except beacon pause and squelch percent)."), NULL);
+
+  
+  label_soundcorr = gtk_label_new (_("sound correction"));
+  gtk_widget_set_name (label_soundcorr, "label_soundcorr");
+  gtk_widget_ref (label_soundcorr);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "label_soundcorr", label_soundcorr,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (label_soundcorr);
+  gtk_table_attach (GTK_TABLE (table_general), label_soundcorr, 0, 1, 1, 2,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_soundcorr), 0, 0.5);
+  gtk_misc_set_padding (GTK_MISC (label_soundcorr), 5, 0);
+
+  soundcorr = gtk_entry_new_with_max_length (10);
+  gtk_widget_set_name (soundcorr, "soundcorr");
+  gtk_widget_ref (soundcorr);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "soundcorr", soundcorr,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (soundcorr);
+  gtk_table_attach (GTK_TABLE (table_general), soundcorr, 1, 2, 1, 2,
+                    (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
+                    (GtkAttachOptions) (0), 3, 3);
+  gtk_entry_set_text (GTK_ENTRY (soundcorr), _("1.0"));
+  gtk_tooltips_set_tip (tooltips, soundcorr, _("Correction factor for the sample rate of your soundcard. 0.9-1.1. You can get it with one the 3 calibration programs dcf77rx, reffreq or ratecal, included in hf, see the hf-howto. You should calibrate for good pactor results."), NULL);
+    
+  
+  label_timecorr = gtk_label_new (_("time correction"));
+  gtk_widget_set_name (label_timecorr, "label_timecorr");
+  gtk_widget_ref (label_timecorr);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "label_timecorr", label_timecorr,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (label_timecorr);
+  gtk_table_attach (GTK_TABLE (table_general), label_timecorr, 0, 1, 2, 3,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_timecorr), 0, 0.5);
+  gtk_misc_set_padding (GTK_MISC (label_timecorr), 5, 0);
+
+  timecorr = gtk_entry_new_with_max_length (10);
+  gtk_widget_set_name (timecorr, "timecorr");
+  gtk_widget_ref (timecorr);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "timecorr", timecorr,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (timecorr);
+  gtk_table_attach (GTK_TABLE (table_general), timecorr, 1, 2, 2, 3,
+                    (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
+                    (GtkAttachOptions) (0), 3, 3);
+  gtk_entry_set_text (GTK_ENTRY (timecorr), _("1.0"));
+  gtk_tooltips_set_tip (tooltips, timecorr, _("Correction factor for the gettimeofday system call. This is only seldom needed if your soundcard can not work full-duplex. You can get it with one the calibration programs dcf77rx or reffreq, included in hf, see the hf-howto."), NULL);
+  
+  
+  label_cpumhzcorr = gtk_label_new (_("cpu rate correction"));
+  gtk_widget_set_name (label_cpumhzcorr, "label_cpumhzcorr");
+  gtk_widget_ref (label_cpumhzcorr);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "label_cpumhzcorr", label_cpumhzcorr,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (label_cpumhzcorr);
+  gtk_table_attach (GTK_TABLE (table_general), label_cpumhzcorr, 0, 1, 3, 4,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_cpumhzcorr), 0, 0.5);
+  gtk_misc_set_padding (GTK_MISC (label_cpumhzcorr), 5, 0);
+
+  
+
+  cpumhzcorr = gtk_entry_new_with_max_length (10);
+  gtk_widget_set_name (cpumhzcorr, "cpumhzcorr");
+  gtk_widget_ref (cpumhzcorr);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "cpumhzcorr", cpumhzcorr,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (cpumhzcorr);
+  gtk_table_attach (GTK_TABLE (table_general), cpumhzcorr, 1, 2, 3, 4,
+                    (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
+                    (GtkAttachOptions) (0), 3, 3);
+  gtk_entry_set_text (GTK_ENTRY (cpumhzcorr), _("0"));
+  gtk_tooltips_set_tip (tooltips, cpumhzcorr, _("Enter here your real cpu MHZ rate. This is only very very seldom needed if a) your soundcard can not work full-duplex and b) your gettimeofday system call does not work. You can get it with one the calibration programs dcf77rx or reffreq, included in hf, see the hf-howto."), NULL);
+  
+  label_soundcard = gtk_label_new (_("Sound Device"));
+  gtk_widget_set_name (label_soundcard, "label_soundcard");
+  gtk_widget_ref (label_soundcard);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "label_soundcard", label_soundcard,(GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (label_soundcard);
+  gtk_table_attach (GTK_TABLE (table_general),label_soundcard, 0, 1, 4, 5,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_soundcard), 0, 0.5);
+  gtk_misc_set_padding (GTK_MISC (label_soundcard), 5, 0);
+
+  soundcard = gtk_combo_new ();
+  gtk_widget_set_name (soundcard, "soundcard");
+  gtk_widget_ref (soundcard);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "soundcard", soundcard,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (soundcard);
+  gtk_table_attach (GTK_TABLE (table_general), soundcard, 1, 2, 4, 5,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
+                    (GtkAttachOptions) (0), 3, 3);
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("/dev/dsp0"));
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("/dev/dsp1"));  
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("/dev/dsp2"));  
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("/dev/dsp3"));  
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("hw:0"));  
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("hw:1"));
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("hw:2"));
+  soundcard_items = g_list_append (soundcard_items, (gpointer) _("hw:3"));
+  gtk_combo_set_popdown_strings (GTK_COMBO (soundcard), soundcard_items);
+  g_list_free (soundcard_items);
+
+  soundcardvalue = GTK_COMBO (soundcard)->entry;
+  gtk_widget_set_name (soundcardvalue, "soundcardvalue");
+  gtk_widget_ref (soundcardvalue);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "soundcardvalue", soundcardvalue, (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (soundcardvalue);
+  gtk_entry_set_text (GTK_ENTRY (soundcardvalue), _("/dev/dsp0"));
+  gtk_tooltips_set_tip (tooltips, soundcardvalue, _("/dev/dsp0 is default, first soundcard, and I will use default OSS API code. You can also test the new ALSA code by using hw:0 for the first sound card and so on. See the hf-howto."), NULL);
+  
+  label_serial = gtk_label_new (_("Serial Device"));
+  gtk_widget_set_name (label_serial, "label_serial");
+  gtk_widget_ref (label_serial);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "label_serial", label_serial,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (label_serial);
+  gtk_table_attach (GTK_TABLE (table_general),label_serial, 0, 1, 5, 6,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_serial), 0, 0.5);
+  gtk_misc_set_padding (GTK_MISC (label_serial), 5, 0);
+
+  serial = gtk_combo_new ();
+  gtk_widget_set_name (serial, "serial");
+  gtk_widget_ref (serial);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "serial", serial,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (serial);
+  gtk_table_attach (GTK_TABLE (table_general), serial, 1, 2, 5, 6,
+                    (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
+                    (GtkAttachOptions) (0), 3, 3);
+  serial_items = g_list_append (serial_items, (gpointer) _("/dev/ttyS0"));
+  serial_items = g_list_append (serial_items, (gpointer) _("/dev/ttyS1"));  
+  serial_items = g_list_append (serial_items, (gpointer) _("/dev/ttyS2"));  
+  serial_items = g_list_append (serial_items, (gpointer) _("/dev/ttyS3"));  
+  gtk_combo_set_popdown_strings (GTK_COMBO (serial), serial_items);
+  g_list_free (serial_items);
+
+  serialvalue = GTK_COMBO (serial)->entry;
+  gtk_widget_set_name (serialvalue, "serialvalue");
+  gtk_widget_ref (serialvalue);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "serialvalue", serialvalue,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (serialvalue);
+  gtk_entry_set_text (GTK_ENTRY (serialvalue), _("none"));
+  gtk_tooltips_set_tip (tooltips, serialvalue, _("for ptt. None is default to prevent errors at first start."), NULL);
+  
+  label_kerneloptions = gtk_label_new (_("hfkernel options"));
+  gtk_widget_set_name (label_kerneloptions, "label_kerneloptions");
+  gtk_widget_ref (label_kerneloptions);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "label_kerneloptions", label_kerneloptions,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (label_kerneloptions);
+  gtk_table_attach (GTK_TABLE (table_general), label_kerneloptions, 0, 1, 6, 7,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_kerneloptions), 0, 0.5);
+  gtk_misc_set_padding (GTK_MISC (label_kerneloptions), 5, 0);
+
+  kerneloptions = gtk_entry_new_with_max_length (256);
+  gtk_widget_set_name (kerneloptions, "kerneloptions");
+  gtk_widget_ref (kerneloptions);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "kerneloptions", kerneloptions,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (kerneloptions);
+  gtk_table_attach (GTK_TABLE (table_general), kerneloptions, 1, 2, 6, 7,
+                    (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
+                    (GtkAttachOptions) (0), 3, 3);
+  gtk_entry_set_text (GTK_ENTRY (kerneloptions), _("1.0"));
+  gtk_tooltips_set_tip (tooltips, kerneloptions, _("Normally no entries are needed here. For special options to hfkernel (the real-time sound driver). E.g. -n = no-memory-mapping, -h = halfduplex, -R = no RDTSC. Should be seldom needed, as hfkernel has now a better hardware detection. If hfkernel does not start, experiment with the options at a console first, see the instructions in the hf-howto."), NULL);
+  
   label_beaconpause = gtk_label_new (_("Beacon Pause (sec)"));
   gtk_widget_set_name (label_beaconpause, "label_beaconpause");
   gtk_widget_ref (label_beaconpause);
   gtk_object_set_data_full (GTK_OBJECT (wpar), "label_beaconpause", label_beaconpause,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label_beaconpause);
-  gtk_table_attach (GTK_TABLE (table_general), label_beaconpause, 0, 1, 1, 2,
+  gtk_table_attach (GTK_TABLE (table_general),label_beaconpause, 0, 1, 8, 9,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label_beaconpause), 0, 0.5);
@@ -429,7 +637,7 @@ create_wpar (void)
   gtk_object_set_data_full (GTK_OBJECT (wpar), "beaconpause", beaconpause,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (beaconpause);
-  gtk_table_attach (GTK_TABLE (table_general), beaconpause, 1, 2, 1, 2,
+  gtk_table_attach (GTK_TABLE (table_general), beaconpause, 1, 2, 8, 9,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
                     (GtkAttachOptions) (0), 3, 3);
   beaconpause_items = g_list_append (beaconpause_items, (gpointer) _("  5"));
@@ -447,8 +655,7 @@ create_wpar (void)
   beaconpausevalue = GTK_COMBO (beaconpause)->entry;
   gtk_widget_set_name (beaconpausevalue, "beaconpausevalue");
   gtk_widget_ref (beaconpausevalue);
-  gtk_object_set_data_full (GTK_OBJECT (wpar), "beaconpausevalue", beaconpausevalue,
-                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "beaconpausevalue", beaconpausevalue, (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (beaconpausevalue);
   gtk_entry_set_text (GTK_ENTRY (beaconpausevalue), _("10"));
 
@@ -458,7 +665,7 @@ create_wpar (void)
   gtk_object_set_data_full (GTK_OBJECT (wpar), "label_squelch", label_squelch,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label_squelch);
-  gtk_table_attach (GTK_TABLE (table_general), label_squelch, 0, 1, 2, 3,
+  gtk_table_attach (GTK_TABLE (table_general), label_squelch, 0, 1, 9, 10,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label_squelch), 0, 0.5);
@@ -467,12 +674,10 @@ create_wpar (void)
   squelchpercent = gtk_entry_new_with_max_length (2);
   gtk_widget_set_name (squelchpercent, "squelchpercent");
   gtk_widget_ref (squelchpercent);
-  gtk_object_set_data_full (GTK_OBJECT (wpar), "squelchpercent", squelchpercent,
-                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_object_set_data_full (GTK_OBJECT (wpar), "squelchpercent", squelchpercent, (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (squelchpercent);
-  gtk_table_attach (GTK_TABLE (table_general), squelchpercent, 1, 2, 2, 3,
-                    (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
-                    (GtkAttachOptions) (0), 3, 3);
+  gtk_table_attach (GTK_TABLE (table_general), squelchpercent, 1, 2, 9, 10, (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
+     (GtkAttachOptions) (0), 3, 3);
   gtk_entry_set_text (GTK_ENTRY (squelchpercent), _("7"));
 
 /* FSK mark + space + squelch */
@@ -615,16 +820,18 @@ create_wpar (void)
                     (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
                     (GtkAttachOptions) (0), 3, 3);
   gtk_entry_set_text (GTK_ENTRY (tone), _("880"));
+  gtk_tooltips_set_tip (tooltips, tone, _("pc speaker tone for elbug. 0 for silent."), NULL);
 
   farnsworth = gtk_check_button_new_with_label (_("Farnsworth Spacing"));
   gtk_widget_set_name (farnsworth, "farnsworth");
   gtk_widget_ref (farnsworth);
   gtk_object_set_data_full (GTK_OBJECT (wpar), "farnsworth", farnsworth,
                             (GtkDestroyNotify) gtk_widget_unref);
-  gtk_widget_show (farnsworth);
+  //gtk_widget_show (farnsworth);
   gtk_table_attach (GTK_TABLE (table_cw), farnsworth, 1, 2, 2, 3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 3, 3);
+  gtk_tooltips_set_tip (tooltips, farnsworth, _("not yet implemented..."), NULL);
 
   dtr = gtk_check_button_new_with_label (_("DTR pin for elbug"));
   gtk_widget_set_name (dtr, "dtr");
@@ -686,7 +893,7 @@ create_wpar (void)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 3, 3);
 
-  rttybaudrate = gtk_entry_new_with_max_length (5);
+  rttybaudrate = gtk_entry_new_with_max_length (6);
   gtk_widget_set_name (rttybaudrate, "rttybaudrate");
   gtk_widget_ref (rttybaudrate);
   gtk_object_set_data_full (GTK_OBJECT (wpar), "rttybaudrate", rttybaudrate,
@@ -695,7 +902,7 @@ create_wpar (void)
   gtk_table_attach (GTK_TABLE (table_rtty), rttybaudrate, 1, 2, 0, 1,
                     (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
                     (GtkAttachOptions) (0), 3, 3);
-  gtk_entry_set_text (GTK_ENTRY (rttybaudrate), _("45"));
+  gtk_entry_set_text (GTK_ENTRY (rttybaudrate), _("45.45"));
 
   label_RTTY = gtk_label_new (_("RTTY"));
   gtk_widget_set_name (label_RTTY, "label_RTTY");
