@@ -295,6 +295,14 @@ void fixtext_read()
 	FILE *fix;
 	int i; 
 	char fixfilename[32], fixtextname[8], buf[16];
+
+	static int radiofont_loaded = 0;
+	if (! radiofont_loaded ) {
+	    if ((radiofont = gdk_font_load("7x13")) == NULL)
+		display_status("simple font 7x13 could not be loaded.");
+  	    else radiofont_loaded = 1;
+	}
+
 	for (i = 1; i < 13; i++) {
 	    sprintf(fixfilename, "hf/fix.%02hu", i);
 	    sprintf(fixtextname, "text%hu", i);
@@ -913,10 +921,10 @@ void autotx()
 	    
 		    default:
 		    // here unclear what to do -> do what user did last 	
-			display_status("autotx: did not find last tx mode.");
-			fittingtxcmd = HFAPP_MSG_START_PACTOR_FEQ;
-			display_status ("-> guess and switch to Pactor FEC..."); 
-			break;
+			display_status("autotx: Please select a tx mode.");
+			//fittingtxcmd = HFAPP_MSG_START_PACTOR_FEQ;
+			//display_status ("-> guess and switch to Pactor FEC..."); 
+			return;
 		}
 		break;
 	}
@@ -1108,10 +1116,11 @@ void autorx()
 		    default:
 	    		fittingrxcmd = lastrxcmd;
 			if ( ! fittingrxcmd ) {   
-			    display_status("autorx: did not find last rx mode.");
-			    fittingrxcmd = HFAPP_MSG_START_STANDBY;
-			    display_status 
-			    ("autorx: -> I guess and switch to Pactor Amtor Gtor Standby"); 
+			    display_status("autorx: Please select a rx mode.");
+			    //fittingrxcmd = HFAPP_MSG_START_STANDBY;
+			    //display_status 
+			    //("autorx: -> I guess and switch to Pactor Amtor Gtor Standby"); 
+			    return;
 			} else {
 			    display_status
 				("autorx: switching to last rx mode: %d", fittingrxcmd);
