@@ -34,7 +34,6 @@
 
 #define SQUELCHPOINTS 9
 guint squelch = 0;
-int squelchpercent = 10;
 int squelch_passed_fsk;
 int squelch_passed_cw;
 int squelch_passed_mt36_500_1000;
@@ -436,15 +435,14 @@ static void draw(Spectrum *spec)
 	}
         /* squelch line: */
         {
-        // i make the squelch 20 % for first draw
-	// later squelch is set by click on vruler right side
-                if (! squelch) 
-		    squelch = SPECTRUM_HEIGHT * (100 - squelchpercent) / 100; 
-                i = squelch;
-                //display_status("squelchpercent = %d", squelchpercent);
-                //display_status("squelch = %d", i);
-                segp = squelchline;
-                segp->y1 = segp->y2 = i;
+		squelch = SPECTRUM_HEIGHT * (100 - params.general.squelchpercent) / 100; 
+                /*
+		display_status("spec draw: params.general.squelchpercent = %d", 
+		    params.general.squelchpercent);
+                display_status("spec draw: squelch = %d", i);
+                */
+		segp = squelchline;
+                segp->y1 = segp->y2 = squelch;
                 segp->x1 = 0;
                 segp->x2 = SPECTRUM_WIDTH-1;
 	}
