@@ -68,8 +68,13 @@
 #ifdef HAVE_SYS_SOUNDCARD_H
 #include <sys/soundcard.h>
 #include <sys/ioctl.h>
+#ifdef __linux__
 #include <endian.h>
 #endif
+#ifdef __FreeBSD__
+#include <sys/endian.h>
+#endif
+#endif
 
 #ifdef DISABLE_REALTIME
 #undef HAVE_SCHED_H
