@@ -51,6 +51,11 @@
 #include <endian.h>
 #endif
 
+#ifdef __FreeBSD__
+#include <sys/soundcard.h>
+#include <sys/endian.h>
+#endif
+
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -108,7 +113,12 @@ void output()
 {
         static char head[256],  foot[256];
 	char factors[512];
-	const char *configfilename =  "/etc/hf.conf";
+#ifdef __linux__
+	const char *configfilename =  "/etc/calibrations";
+#endif
+#ifdef __FreeBSD__
+	const char *configfilename =  "/usr/local/etc/calibrations";
+#endif
 	FILE *configfile = NULL;
 	static int prepared = 0;
 
